#include <assert.h>
#include <dos.h>
#include <malloc.h>
#include <stdio.h>
#include "anim256.h"
#include "evntmngr.h"
#include "font.h"
#include "keyboard.h"
#include "mouse.h"
#include "scancode.h"
#include "screen.h"

extern BYTE CurrColor;
extern Mouse mouse;

// CursorRect Constructor
CursorRect::CursorRect(int InitX1, int InitY1, int InitX2, int InitY2,
		int InitColor, BYTE *InitScreen) :
	Rectangle(InitX1, InitY1, InitX2, InitY2, InitColor, NOSAVE_BG,
		InitScreen)
{
} // CursorRect Constructor

void CursorRect::Show()
{
	hlinedraw(X1, Y1, X2, Color, screen);
	hlinedraw(X1, Y2, X2, Color, screen);
	vlinedraw(X1, Y1, Y2, Color, screen);
	vlinedraw(X2, Y1, Y2, Color, screen);
} // CursorRect::Show

// EditGrid Constructor
EditGrid::EditGrid(int InitAcross, int InitDown, int InitX1, int InitY1,
		int InitX2, int InitY2, int InitColor,
		int InitCursColor, BYTE *InitScreen) :
	Grid(InitAcross, InitDown, InitX1, InitY1, InitX2, InitY2,
		InitColor, InitScreen)
{
	EditCursor = new CursorRect(X1, Y1, X1+CellWidth, Y1+CellHeight,
		InitCursColor, InitScreen);
	CursAc = 0;
	CursDn = 0;
} // EditGrid Constructor

void EditGrid::Show()
{
	Grid::Show();
	EditCursor->Show();
} // EditGrid::Show

void EditGrid::ShowCursor()
{
	EditCursor->Show();
} // EditGrid::ShowCursor

void EditGrid::GetCursCoords(int *CX1, int *CY1, int *CX2, int *CY2)
{
	EditCursor->GetCoords(CX1, CY1, CX2, CY2);
} // EditGrid::GetCursCoords

void EditGrid::MoveCurs(int CX1, int CY1, int CX2, int CY2)
{
	EditCursor->MoveTo(CX1, CY1, CX2, CY2);
} // EditGrid::MoveCurs

void EditGrid::HandleEvent(int Event_Type, int Event)
{
	int cX1, cY1, cX2, cY2;
	int OldColor;
	int mX, mY;
	int moved = 0;

	GetCursCoords(&cX1, &cY1, &cX2, &cY2);
	if(Event_Type == MOUSE_EVENT)
	{
		mouse.GetPos(&mX, &mY);
		if((mX >= X1) && (mX <= X2) && (mY >= Y1) && (mY <= Y2))
		{
			while (mY < cY1)
			{
				cY1 -= CellHeight;
				cY2 -= CellHeight;
				CursDn --;
				moved = 1;
			}
			while (mY > cY2)
			{
				cY1 += CellHeight;
				cY2 += CellHeight;
				CursDn++;
				moved = 1;
			}
			while (mX < cX1)
			{
				cX1 -= CellWidth;
				cX2 -= CellWidth;
				CursAc--;
				moved = 1;
			}
			while (mX > cX2)
			{
				cX1 += CellWidth;
				cX2 += CellWidth;
				CursAc++;
				moved = 1;
			}
			if(moved)
			{
				OldColor = EditCursor->GetColor();
				EditCursor->SetColor(GetColor());
				EditCursor->Show();
				EditCursor->MoveTo(cX1, cY1, cX2, cY2);
				EditCursor->SetColor(OldColor);
				EditCursor->Show();
			}
		}
	}
	else if (Event_Type == KEYBOARD_EVENT)
	{
		OldColor = EditCursor->GetColor();
		EditCursor->SetColor(GetColor());
		EditCursor->Show();
		switch(Event)
		{
			case SCODELEFT:
				if(CursAc)
				{
					cX1 -= CellWidth;
					cX2 -= CellWidth;
					CursAc --;
				}
				break;
			case SCODERIGHT:
				if(CursAc < Across-1)
				{
					cX1 += CellWidth;
					cX2 += CellWidth;
					CursAc ++;
				}
				break;
			case SCODEUP:
				if(CursDn)
				{
					cY1 -= CellHeight;
					cY2 -= CellHeight;
					CursDn --;
				}
				break;
			case SCODEDOWN:
				if(CursDn < Down-1)
				{
					cY1 += CellHeight;
					cY2 += CellHeight;
					CursDn ++;
				}
				break;
		}
		EditCursor->MoveTo(cX1, cY1, cX2, cY2);
		EditCursor->SetColor(OldColor);
		EditCursor->Show();
	}
} // EditGrid::HandleEvent

void EditGrid::HandleEvent(int Event_Type, int Event, int HideColor)
{
	int cX1, cY1, cX2, cY2;
	int OldColor;
	int mX, mY;
	int moved = 0;

	GetCursCoords(&cX1, &cY1, &cX2, &cY2);
	if(Event_Type == MOUSE_EVENT)
	{
		mouse.GetPos(&mX, &mY);
		if((mX >= X1) && (mX <= X2) && (mY >= Y1) && (mY <= Y2))
		{
			while (mY < cY1)
			{
				cY1 -= CellHeight;
				cY2 -= CellHeight;
				CursDn --;
				moved = 1;
			}
			while (mY > cY2)
			{
				cY1 += CellHeight;
				cY2 += CellHeight;
				CursDn++;
				moved = 1;
			}
			while (mX < cX1)
			{
				cX1 -= CellWidth;
				cX2 -= CellWidth;
				CursAc--;
				moved = 1;
			}
			while (mX > cX2)
			{
				cX1 += CellWidth;
				cX2 += CellWidth;
				CursAc++;
				moved = 1;
			}
			if(moved)
			{
				OldColor = EditCursor->GetColor();
				EditCursor->SetColor(HideColor);
				EditCursor->Show();
				EditCursor->MoveTo(cX1, cY1, cX2, cY2);
				EditCursor->SetColor(OldColor);
				EditCursor->Show();
			}
		}
	}
	else if (Event_Type == KEYBOARD_EVENT)
	{
		OldColor = EditCursor->GetColor();
		EditCursor->SetColor(HideColor);
		EditCursor->Show();
		switch(Event)
		{
			case SCODELEFT:
				if(CursAc)
				{
					cX1 -= CellWidth;
					cX2 -= CellWidth;
					CursAc --;
				}
				break;
			case SCODERIGHT:
				if(CursAc < Across-1)
				{
					cX1 += CellWidth;
					cX2 += CellWidth;
					CursAc ++;
				}
				break;
			case SCODEUP:
				if(CursDn)
				{
					cY1 -= CellHeight;
					cY2 -= CellHeight;
					CursDn --;
				}
				break;
			case SCODEDOWN:
				if(CursDn < Down-1)
				{
					cY1 += CellHeight;
					cY2 += CellHeight;
					CursDn ++;
				}
				break;
		}
		EditCursor->MoveTo(cX1, cY1, cX2, cY2);
		EditCursor->SetColor(OldColor);
		EditCursor->Show();
	}
} // EditGrid::HandleEvent

EditGrid::~EditGrid()
{
	delete EditCursor;
} // EditGrid Destructor

// PaletteGrid Constructor
PaletteGrid::PaletteGrid(int InitAcross, int InitDown, int InitX1,
		int InitY1, int InitX2, int InitY2, int InitColor,
		int InitCursColor, BYTE *InitRGB, BYTE *InitScreen) :
	EditGrid(InitAcross, InitDown, InitX1, InitY1, InitX2, InitY2,
		InitColor, InitCursColor, InitScreen),
	Palette(InitRGB)
{
} // PaletteGrid Constructor

void PaletteGrid::Show()
{
	int k=0;

	EditGrid::Show();
	mouse.MaybeHide();
	for(int i=0;i<Down;i++)
		for(int j=0;j<Across;j++)
			boxdraw((X1+1)+(j*CellWidth), (Y1+1)+(i*CellHeight),
				(X1+CellWidth)+(j*CellWidth)-1,
				(Y1+CellHeight)+(i*CellHeight)-1, k++, screen);
	mouse.MaybeShow();
	CurrColor = 0;
} // PaletteGrid::Show

void PaletteGrid::MoveCursor(int Ac, int Dn)
{
	int cX1, cY1, cX2, cY2;

	EditGrid::GetCoords(&cX1, &cY1, &cX2, &cY2);
	int OldColor = EditCursor->GetColor();
	EditCursor->SetColor(GetColor());
	EditCursor->Show();
	EditCursor->MoveTo(cX1+Ac*CellWidth, cY1+Dn*CellHeight,
		cX1+Ac*CellWidth+CellWidth, cY1+Dn*CellHeight+CellHeight);
	EditCursor->SetColor(OldColor);
	EditCursor->Show();
	CursAc = Ac;
	CursDn = Dn;
} // PaletteGrid::MoveCursor

void PaletteGrid::HandleEvent(int Event_Type, int Event)
{
	int cX1, cY1, cX2, cY2, mX, mY;
	int moved, OldColor;

	EditGrid::GetCursCoords(&cX1, &cY1, &cX2, &cY2);

	if( ((Event_Type == MOUSE_EVENT) &&
		((Event & LBUTTONDOWN)||(Event & LBUTTONHELD)))
		|| ((Event_Type == KEYBOARD_EVENT) && (Event == SCODEENTER)) )
	{
		mouse.GetPos(&mX, &mY);
		while (mY < cY1)
		{
			cY1 -= CellHeight;
			cY2 -= CellHeight;
			CursDn --;
			moved = 1;
		}
		while (mY > cY2)
		{
			cY1 += CellHeight;
			cY2 += CellHeight;
			CursDn++;
			moved = 1;
		}
		while (mX < cX1)
		{
			cX1 -= CellWidth;
			cX2 -= CellWidth;
			CursAc--;
			moved = 1;
		}
		while (mX > cX2)
		{
			cX1 += CellWidth;
			cX2 += CellWidth;
			CursAc++;
			moved = 1;
		}
		if(moved)
		{
			mouse.MaybeHide();
			OldColor = EditCursor->GetColor();
			EditCursor->SetColor(GetColor());
			EditCursor->Show();
			EditCursor->MoveTo(cX1, cY1, cX2, cY2);
			EditCursor->SetColor(OldColor);
			EditCursor->Show();
			CurrColor = CursAc + (CursDn * Across);
			mouse.MaybeShow();
		}
	}
	else if(Event_Type == KEYBOARD_EVENT)
	{
		EditGrid::HandleEvent(Event_Type, Event);
		CurrColor = CursAc + (CursDn * Across);
	}
} // PaletteGrid::HandleEvent

// SpriteGrid Constructor
SpriteGrid::SpriteGrid(int InitAcross, int InitDown, int InitX1, int InitY1,
		int InitX2, int InitY2, int InitColor,
		int InitCursColor, BYTE *InitImage,
		PaletteGrid *InitPaletteGrid, BYTE *InitScreen) :
	EditGrid(InitAcross, InitDown, InitX1, InitY1, InitX2, InitY2,
		InitColor, InitCursColor, InitScreen)
{
	Colors = InitImage;
	using_displayer = 0;
	palettegrid = InitPaletteGrid;
	gridon = 1;
	color = InitColor;
} // SpriteGrid Constructor

void SpriteGrid::Refresh()
{
	int i,j;
	int k=0;
	mouse.MaybeHide();
	for(i=0;i<Down;i++)
		for(j=0;j<Across;j++)
			boxdraw((X1+1)+(j*CellWidth), (Y1+1)+(i*CellHeight),
				(X1+CellWidth)+(j*CellWidth)-gridon,
				(Y1+CellHeight)+(i*CellHeight)-gridon,
				Colors[k++], screen);
	mouse.MaybeShow();
} // SpriteGrid::Refresh

void SpriteGrid::Show()
{
	EditGrid::Show();
	Refresh();
	ShowCursor();
} // SpriteGrid::Show

void SpriteGrid::HandleEvent(int Event_Type, int Event)
{
	int cX1, cY1, cX2, cY2;

	if(gridon)
		EditGrid::HandleEvent(Event_Type, Event);
	else
		EditGrid::HandleEvent(Event_Type, Event,
			Colors[CursAc+CursDn*Across]);

	if( ((Event_Type == MOUSE_EVENT) &&
		((Event & LBUTTONDOWN)||(Event & LBUTTONHELD)))
		|| ((Event_Type == KEYBOARD_EVENT)
			&& ((Event == SCODEENTER) || (Event == SCODESPACE))) )
	{
		GetCursCoords(&cX1, &cY1, &cX2, &cY2);
		mouse.MaybeHide();
		boxdraw(cX1+1, cY1+1, cX2-1, cY2-1, CurrColor, screen);
		mouse.MaybeShow();
		Colors[CursAc+CursDn*Across] = CurrColor;
		if(using_displayer)
			spritedisplayer->PutNewPixel(CursAc, CursDn);
	}
	if(((Event_Type == MOUSE_EVENT) &&
		((Event & RBUTTONDOWN) || (Event & RBUTTONUP)))
		|| ((Event_Type == KEYBOARD_EVENT) && (Event == SCODEESC)))
	{
		CurrColor = Colors[CursAc+CursDn*Across];
		palettegrid->MoveCursor(CurrColor%32,CurrColor/32);
	}
} // SpriteGrid::HandleEvent

void SpriteGrid::AddDisplayer(SpriteDisplayer *InitSpriteDisplayer)
{
	using_displayer = 1;
	spritedisplayer = InitSpriteDisplayer;
} // SpriteGrid::AddDisplayer

void SpriteGrid::Invert()
{
	int i,j;
	int k=0;

	for(i=0;i<Down;i++)
		for(j=0;j<Across;j++)
			Colors[k++] ^= 0xFF;
	Refresh();
} // SpriteGrid::Invert

void SpriteGrid::ColorFill(BYTE FillColor)
{
	int i,j;
	int k=0;

	for(i=0;i<Down;i++)
		for(j=0;j<Across;j++)
			Colors[k++] = FillColor;
	Refresh();
} // SpriteGrid::ColorFill

void SpriteGrid::Clear()
{
	ColorFill(0);
} // SpriteGrid::Clear

void SpriteGrid::ScrollLeft()
{
	BYTE *temp1;
	BYTE *temp2;
	BYTE temp;
	int i;
	int j;

	for(i=0;i<Down;i++)
	{
		temp1 = Colors+(i*Across);
		temp2 = temp1+1;
		temp = *temp1;
		for(j=0;j<Across;j++,temp1++,temp2++)
			*temp1 = *temp2;
		Colors[i*Across+Across-1] = temp;
	}
	Refresh();
} // SpriteGrid::ScrollLeft

void SpriteGrid::ScrollRight()
{
	BYTE *temp1;
	BYTE *temp2;
	BYTE temp;
	int i;
	int j;

	for(i=0;i<Down;i++)
	{
		temp1 = Colors+(i*Across+Across-1);
		temp2 = temp1-1;
		temp = *temp1;
		for(j=0;j<Across;j++,temp1--,temp2--)
			*temp1 = *temp2;
		Colors[i*Across] = temp;
	}
	Refresh();
} // SpriteGrid::ScrollRight

void SpriteGrid::ScrollUp()
{
	BYTE *temp1;
	BYTE *temp2;
	BYTE temp;
	int i, j;

	for(i=0;i<Across;i++)
	{
		temp1 = Colors + i;
		temp2 = temp1 + Across;
		temp = *temp1;
		for(j=0;j<Down;j++,temp1+=Across,temp2+=Across)
			*temp1 = *temp2;
		Colors[Across*(Down-1)+i] = temp;
	}
	Refresh();
} // SpriteGrid::ScrollUp

void SpriteGrid::ScrollDown()
{
	BYTE *temp1;
	BYTE *temp2;
	BYTE temp;
	int i, j;

	for(i=0;i<Across;i++)
	{
		temp1 = Colors + i + Across*(Down-1) - Across;
		temp2 = Colors + i + Across*(Down-1);
		temp = *temp2;
		for(j=0;j<Down;j++,temp1-=Across,temp2-=Across)
			*temp2 = *temp1;
		Colors[i] = temp;
	}
	Refresh();
} // SpriteGrid::Down

void SpriteGrid::HFlip()
{
	BYTE *temp1;
	BYTE *temp2;
	BYTE temp;
	int i;

	for(i=0;i<Down;i++)
	{
		temp1 = Colors+(i*Across);
		temp2 = temp1+Across-1;
		while(temp1 < temp2)
		{
			temp = *temp1;
			*temp1 = *temp2;
			*temp2 = temp;
			temp1++; temp2--;
		}
	}
	Refresh();
} // SpriteGrid::HFlip

void SpriteGrid::VFlip()
{
	BYTE *temp1;
	BYTE *temp2;
	BYTE temp;
	int i;

	for(i=0;i<Across;i++)
	{
		temp1 = Colors + i;
		temp2 = temp1 + Across*Down - Across;
		while(temp1 < temp2)
		{
			temp = *temp1;
			*temp1 = *temp2;
			*temp2 = temp;
			temp1 += Across;
			temp2 -= Across;
		}
	}
	Refresh();
} // SpriteGrid::VFlip

void SpriteGrid::FillGrid(BYTE *NewColors)
{
	Colors = NewColors;
	Refresh();
} // SpriteGrid::FillGrid

void SpriteGrid::ReplaceColor(BYTE NewColor)
{
	BYTE OldColor;
	int i;

	OldColor = Colors[CursAc+CursDn*Across];
	for(i=0;i<Across*Down;i++)
		if(Colors[i] == OldColor)
			Colors[i] = NewColor;
	Refresh();
	if(using_displayer)
		spritedisplayer->RefreshActive();
} // SpriteGrid::FillGrid

void SpriteGrid::GridOn()
{
	int cx1, cx2, cy1, cy2;

	if(gridon)
		return;
	GetCursCoords(&cx1, &cy1, &cx2, &cy2);
	MoveCurs(cx1-1, cy1-1, cx2, cy2);
	SetColor(color);
	gridon = 1;
	Show();
} // SpriteGrid:GridOn

void SpriteGrid::GridOff()
{
	int cx1, cx2, cy1, cy2;

	if(!gridon)
		return;
	GetCursCoords(&cx1, &cy1, &cx2, &cy2);
	MoveCurs(cx1+1, cy1+1, cx2, cy2);
	SetColor(0);
	gridon = 0;
	Show();
} // SpriteGrid::GridOff

// SpriteDisplayer Constructor
SpriteDisplayer::SpriteDisplayer(int InitX1, int InitY1, int InitX2,
		int InitY2, int InitColor, int InitTopColor,
		int InitBottomColor, BYTE *InitScreen,
		FontStruct *InitFontPtr,
		int InitTextColor, Sprite *InitSprite,
		SpriteGrid *InitSpriteGrid, int InitLines) :
	TextButton(InitX1, InitY1, InitX2, InitY2, InitColor, InitTopColor,
		InitBottomColor, NOSAVE_BG, InitScreen, "", InitFontPtr,
		InitTextColor, CENTER_TEXT, CENTER_TEXT, InitLines)
{
	int x1,y1,x2,y2;
	int SprWidth, SprHeight;
	int XSpace, YSpace;
	int XDist, YDist;           // Distances between buttons
	int YEdgeDist;              // Minimum distance to edge
	int XEdgeDist;
	int i;
	char PGUPText[2];
	char PGDNText[2];
	int end_of_row = 0;

	XDist = InitFontPtr->width * 2 - 1;
	YDist = InitFontPtr->height * 2 - 1;
	XEdgeDist = InitFontPtr->width * 2 - 1;
	YEdgeDist = InitFontPtr->height + 1;

	NumSprites = InitSprite->GetNumSprites();
	active_sprite = 0;
	sprite = InitSprite;
	spritegrid = InitSpriteGrid;
	spritebuttons = new Button *[NumSprites];
	InitSprite->GetDimensions(&SprWidth, &SprHeight);
	PGUPText[0] = 0x18;    // Up arrow
	PGUPText[1] = 0;
	PGDNText[0] = 0x19;    // Down arrow
	PGDNText[1] = 0;
	PGUPButton = new TextButton(X2-Lines-4-FontPtr->width,Y1+Lines,
		X2-Lines,Y1+Lines+4+FontPtr->height,
		InitColor,InitTopColor,InitBottomColor,SAVE_BG,InitScreen,
		PGUPText, InitFontPtr, InitTextColor, CENTER_TEXT, CENTER_TEXT,
		InitLines);
	PGDNButton = new TextButton(X2-Lines-4-FontPtr->width,
		Y2-Lines-4-FontPtr->height, X2-Lines,Y2-Lines,
		InitColor,InitTopColor,InitBottomColor,SAVE_BG,InitScreen,
		PGDNText, InitFontPtr, InitTextColor, CENTER_TEXT, CENTER_TEXT,
		InitLines);

	XSpace = SprWidth + Lines + Lines;        // Dimensions of
	YSpace = SprHeight + Lines + Lines;       //    the button itself

	x1 = X1 + YEdgeDist;                      // Coordinates
	y1 = Y1 + YEdgeDist;                      //    of
	x2 = x1 + XSpace - 1;                     //    first
	y2 = y1 + YSpace - 1;                     //    button

	max_num_rows = 1;
	max_num_on_row = 0;
	num_pages = 1;

	for(i=0;i<NumSprites;i++)
	{
		spritebuttons[i] = new Button(x1,y1,x2,y2,
			Color,TopColor,BotColor,NOSAVE_BG,screen,Lines);
		if(i==NumSprites-1)
			break;
		x1 += (XSpace + XDist);
		x2 += (XSpace + XDist);
		if(!end_of_row)
			max_num_on_row++;
		if(x2 > (X2 - XEdgeDist))   // If this button will be past the
		{                               // edge, start next row
			end_of_row = 1;
			x1 = X1 + YEdgeDist;
			x2 = x1 + XSpace - 1;
			y1 += (YSpace + YDist);
			y2 += (YSpace + YDist);
			if(num_pages == 1)
				max_num_rows++;
			if(y2 > (Y2 - YEdgeDist))   // If next row will be past the
			{                               //    bottom edge of panel
				y1 = Y1 + YEdgeDist;        // Start next panel of buttons
				y2 = y1 + YSpace - 1;
				if(num_pages == 1)
					max_num_rows--;
				num_pages++;
			}
		}
	}
	if(!end_of_row)
	{
		max_num_on_row = NumSprites-1;
		while(x2 < (X2 - XEdgeDist))
		{
			x2 += (XSpace + XDist);
			max_num_on_row++;
		}
	}
	// Num on page = how many will fit across * how many will fit down
	num_on_page = max_num_on_row * max_num_rows;
	page = 0;
} // SpriteDisplayer Constructor

void SpriteDisplayer::Show()
{
	int NumSprites;
	int x1,y1,x2,y2;
	char Number[3];
	int i, j;

	TextButton::Show();

	NumSprites = sprite->GetNumSprites();

	for(i=(page*num_on_page),j=0;;i++,j++)
	{
		if((i==NumSprites) || (j==num_on_page))
			break;
		spritebuttons[i]->Show();
		spritebuttons[i]->GetCoords(&x1, &y1, &x2, &y2);
		sprite->put(i, x1+Lines, y1+Lines, screen);
		sprintf(Number, "%d", i+1);
		GraphString(FontPtr, x1, y2+2, TextColor, Number, screen);
	}
	spritebuttons[active_sprite]->Push();
	PGUPButton->Hide();
	PGDNButton->Hide();
	if(page)
		PGUPButton->Show();
	if(page<num_pages-1)
		PGDNButton->Show();
} // SpriteDisplayer::Show

void SpriteDisplayer::RefreshActive()
{
	int x1, y1, x2, y2;

	spritebuttons[active_sprite]->GetCoords(&x1, &y1, &x2, &y2);
	sprite->put(active_sprite, x1+Lines, y1+Lines, screen);
} // SpriteDisplayer::RefreshActive

void SpriteDisplayer::HandleEvent(int Event_Type, int Event)
{
	int x1, y1, x2, y2;
	int mx,my;
	int NumSprites = sprite->GetNumSprites();
	int i;
	int calc;
	int UX1, UX2, UY1, UY2, DX1, DX2, DY1, DY2;

	if( ((Event_Type == MOUSE_EVENT)
		&& ((Event & LBUTTONDOWN)||(Event & LBUTTONHELD)
		||(Event & LBUTTONUP)) )
		|| ((Event_Type == KEYBOARD_EVENT) && (Event == SCODEENTER)) )
	{
		mouse.GetPos(&mx, &my);
		calc = page*num_on_page;
		for(i=calc;i<min(num_on_page+calc, NumSprites);i++)
		{
			spritebuttons[i]->GetCoords(&x1, &y1, &x2, &y2);
			if( (mx>=x1) && (mx<=x2) && (my>=y1) && (my<=y2) )
			{
				if(i!=active_sprite)
				{
					spritebuttons[active_sprite]->Release();
					spritebuttons[i]->Push();
					active_sprite = i;
					spritegrid->FillGrid(sprite->GetSpriteImage(i));
				}
				break;
			}
		}
		PGUPButton->GetCoords(&UX1, &UY1, &UX2, &UY2);
		if( (mx>=UX1) && (mx<=UX2) && (my>=UY1) && (my<=UY2) && (page) )
		{
			if((Event & LBUTTONDOWN)||(Event & LBUTTONHELD)
				||(Event & LBUTTONUP))
			{
				if(Event & LBUTTONUP)
				{
					mouse.MaybeHide();
					if(page)
						PGUPButton->Hide();
					if(page<(num_pages-1))
						PGDNButton->Hide();
					page--;
					active_sprite -= num_on_page;
					spritegrid->FillGrid(
						sprite->GetSpriteImage(active_sprite));
					PGUPButton->Hide();
					PGDNButton->Hide();
					ColorHide(Color);
					Show();
					mouse.MaybeShow();
				}
				else
					PGUPButton->Push();
			}
		}
		else if(page)
			PGUPButton->Release();
		PGDNButton->GetCoords(&DX1, &DY1, &DX2, &DY2);
		if( (mx>=DX1) && (mx<=DX2) && (my>=DY1) && (my<=DY2)
			&& (page<num_pages-1) )
		{
			if((Event & LBUTTONDOWN)||(Event & LBUTTONHELD)
				||(Event & LBUTTONUP))
			{
				if(Event & LBUTTONUP)
				{
					mouse.MaybeHide();
					if(page)
						PGUPButton->Hide();
					if(page<(num_pages-1))
						PGDNButton->Hide();
					page++;
					active_sprite += num_on_page;
					if(active_sprite>(NumSprites-1))
						active_sprite = (NumSprites-1);
					spritegrid->FillGrid(
						sprite->GetSpriteImage(active_sprite));
					PGUPButton->Hide();
					PGDNButton->Hide();
					ColorHide(Color);
					Show();
					mouse.MaybeShow();
				}
				else
					PGDNButton->Push();
			}
		}
		else if(page<num_pages-1)
			PGDNButton->Release();
	}
	else if(Event_Type == KEYBOARD_EVENT)
	{
		mouse.MaybeHide();
		switch(Event)
		{
			case SCODEPGUP:
				if(page)
				{
					if(page<(num_pages-1))
						PGDNButton->Hide();
					PGUPButton->Hide();
					page--;
					active_sprite -= num_on_page;
					spritegrid->FillGrid(
						sprite->GetSpriteImage(active_sprite));
					PGUPButton->Hide();
					PGDNButton->Hide();
					ColorHide(Color);
					Show();
				}
				break;
			case SCODEPGDN:
				if(page<(num_pages-1))
				{
					if(page)
						PGUPButton->Hide();
					PGDNButton->Hide();
					page++;
					active_sprite += num_on_page;
					if(active_sprite>(NumSprites-1))
						active_sprite = (NumSprites-1);
					spritegrid->FillGrid(
						sprite->GetSpriteImage(active_sprite));
					PGUPButton->Hide();
					PGDNButton->Hide();
					ColorHide(Color);
					Show();
				}
				break;
		}
		mouse.MaybeShow();
	}
} // SpriteDisplayer::HandleEvent

void SpriteDisplayer::PutNewPixel(int PX, int PY)
{
	int bx1, by1, bx2, by2;
	unsigned int offset;
	int totY;

	spritebuttons[active_sprite]->GetCoords(&bx1, &by1, &bx2, &by2);
	// you can find a value, n*320, by adding n*256 + n*64, which is
	//     (n << 8) + (n << 6).  Thus, no multiplication is done,
	//     speeding up the calculation.
	// This next line adds the top-left y-coordinate of the current
	//      sprite button, the y-coordinate into the actual sprite the
	//      new pixel is to go, the lines around the button, and finally
	//      all the x-coordinates to reach the final pixel location
	totY = by1 + PY + Lines;
	offset = (totY << 8) + (totY << 6) + (Lines + bx1 + PX);
	screen[offset] = CurrColor;
} // SpriteDisplayer::PutNewPixel

SpriteDisplayer::~SpriteDisplayer()
{
	int i;

	delete PGUPButton;
	delete PGDNButton;
	for(i=0;i<NumSprites;i++)
		delete spritebuttons[i];
	delete spritebuttons;
} // SpriteDisplayer destructor

