#ifndef _ANIM256_H
#define _ANIM256_H

#include "font.h"
#include "menu.h"
#include "mycolors.h"
#include "shapes.h"
#include "sprite.h"

#ifndef min
#define max(a,b)    (((a) > (b)) ? (a) : (b))
#define min(a,b)    (((a) < (b)) ? (a) : (b))
#endif

#ifndef BYTE
typedef unsigned char	BYTE;
typedef unsigned int	WORD;
#endif

// Declaration of CursorRect class
class CursorRect : public Rectangle
{
	private:
	public:
		CursorRect(int InitX1, int InitY1, int InitX2, int InitY2,
			int InitColor, BYTE *InitScreen);
		void Show();
}; // CursorRect class declaration

class EditGrid : public Grid
{
	protected:
		int CursAc, CursDn;
		CursorRect *EditCursor;
	public:
		EditGrid(int InitAcross, int InitDown, int InitX1, int InitY1,
			int InitX2, int InitY2, int InitColor, int InitCursColor,
			BYTE *InitScreen);
		void Show();
		void ShowCursor();
		void HandleEvent(int Event_Type, int Event);
		void HandleEvent(int Event_Type, int Event, int HideColor);
		void GetCursCoords(int *CX1, int *CY1, int *CX2, int *CY2);
		void MoveCurs(int CX1, int CY1, int CX2, int CY2);
		~EditGrid();
}; // EditGrid Constructor

// PaletteGrid Class Declaration
class PaletteGrid : public EditGrid, public Palette
{
	public:
		PaletteGrid(int InitAcross, int InitDown, int InitX1, int InitY1,
			int InitX2, int InitY2, int InitColor, int InitCursColor,
			BYTE *InitRGB, BYTE *InitScreen);
		void Show();
		void MoveCursor(int Ac, int Dn);
		void HandleEvent(int Event_Type, int Event);
}; // PaletteGrid Class declaration

class SpriteGrid : public EditGrid
{
	friend class SpriteDisplayer;
	private:
		int using_displayer;
		SpriteDisplayer *spritedisplayer;
		BYTE *Colors;
		PaletteGrid *palettegrid;
		int gridon;
		int color;
	public:
		SpriteGrid(int InitAcross, int InitDown, int InitX1, int InitY1,
			int InitX2, int InitY2, int InitColor, int InitCursColor,
			BYTE *InitImage, PaletteGrid *InitPaletteGrid, BYTE *InitScreen);
		void Refresh();
		void Show();
		void HandleEvent(int Event_Type, int Event);
		void FillGrid(BYTE *NewColors);
		void ColorFill(BYTE FillColor);
		void Clear();
		void Invert();
		void ScrollLeft();
		void ScrollRight();
		void ScrollUp();
		void ScrollDown();
		void HFlip();
		void VFlip();
		void ReplaceColor(BYTE NewColor);
		void AddDisplayer(SpriteDisplayer *InitSpriteDisplayer);
		void GridOn();
		void GridOff();
}; // SpriteGrid Constructor

// SpriteDisplayer class declaration
class SpriteDisplayer : public TextButton
{
	private:
		int NumSprites;
		int num_on_page;
		int max_num_on_row;                        // (per panel)
		int max_num_rows;                          // (per panel)
		int num_pages;
		int page;
		int active_sprite;
		Sprite *sprite;
		TextButton *PGUPButton;
		TextButton *PGDNButton;
		SpriteGrid *spritegrid;
		Button **spritebuttons;
	public:
		SpriteDisplayer(int InitX1, int InitY1, int InitX2, int InitY2,
			int InitColor, int InitTopColor, int InitBottomColor,
			BYTE *InitScreen, FontStruct *InitFontPtr,
			int InitTextColor, Sprite *InitSprite,
			SpriteGrid *InitSpriteGrid,	int InitLines = DEFAULT_LINES);
		void Show();
		void RefreshActive();
		void HandleEvent(int Event_Type, int Event);
		void PutNewPixel(int PX, int PY);
		int GetActiveSprite()
		{ return active_sprite; };
		~SpriteDisplayer();
}; // SpriteDisplayer class declaration

#endif
