#include <dos.h>
#include "animate.h"
#include "evntmngr.h"
#include "scancode.h"
#include "wait.h"

extern int event_mask;

Animator::Animator(Sprite *InitSprite, int **InitSequences,
		int InitNumSequences, int *InitSequenceLens,
		int *InitSequenceKeyCodes, BYTE *InitScreen)
{
	sprite = InitSprite;
	screen = InitScreen;
	sequences = InitSequences;
	sequence_lens = InitSequenceLens;
	num_sequences = InitNumSequences;
	sequence_key_codes = InitSequenceKeyCodes;
	dx = 0;
	dy = 0;
	wait_time = DEFAULT_WAIT_TIME;
} // Animator::Animator

void Animator::QuickAnimate()
{
	int done=0;
	int event_type, event;
	int counter = 0;
	int len = sequence_lens[0];
	int calcx, calcy;
	int width, height;

	sprite->GetDimensions(&width, &height);

	calcx = (MAXX-width)/2;
	calcy = (MAXY-height)/2;

	sprite->SetNewCoords(calcx,calcy);
	sprite->puttransparent(sequences[0][0], calcx, calcy, screen);

	do {
		sprite->Move(dx, dy, sequences[0][counter++]);
		if(counter>=len)
			counter = 0;
		if(wait_time)
			Wait((unsigned long int)wait_time);
		getevent(event_mask, &event_type, &event);
		if(event_type == KEYBOARD_EVENT)
		{
			switch(event)
			{
				case SCODEESC:
					sprite->erase(screen);
					done = 1;
					break;
				case SCODEKMINUS: case SCODEMINUS: case SCODEUNDER:
					wait_time += WAIT_50th;
					if(wait_time > WAIT_SEC5 )
					wait_time = WAIT_SEC5;
					break;
				case SCODEKPLUS: case SCODEPLUS: case SCODEEQUALS:
					wait_time -= WAIT_50th;
					if(wait_time < 0)
					wait_time = 0;
					break;
				case SCODECS:
					wait_time = DEFAULT_WAIT_TIME;
					break;
			} // end of switch event
		} // end of if event = KEYBOARD
	} while(!done);
} // Animator::QuickAnimate

void Animator::SetNewSequences(int **NewSequences, int NewNumSequences,
	 int *NewSequenceLens, int *NewSequenceKeyCodes)
{
	sequences = NewSequences;
	num_sequences = NewNumSequences;
	sequence_lens = NewSequenceLens;
	sequence_key_codes = NewSequenceKeyCodes;
	wait_time = WAIT_HALFSEC;
} // Animator::SetNewSequences

void Animator::FullAnimate()
{
	int done=0;
	int event_type, event;
	int counter = 0;
	int calcx, calcy;
	int tempdx = 5, tempdy = 5;
	int width, height;
	int j;
	int seq_num;

	for(j=4;j>=0;j--)
	{
		if(sequence_lens[j])
		{
			seq_num = j;
			break;
		}
	}

	sprite->GetDimensions(&width, &height);

	calcx = (MAXX-width)/2;
	calcy = (MAXY-height)/2;

	sprite->SetNewCoords(calcx,calcy);
	sprite->puttransparent(sequences[seq_num][0], calcx, calcy, screen);

	do {
		if(counter>=sequence_lens[seq_num])
			counter = 0;
		if(sequence_lens[seq_num])
			sprite->Move(dx, dy, sequences[seq_num][counter++]);
		if(wait_time)
			Wait((unsigned long int)wait_time);
		getevent(event_mask, &event_type, &event);
		if(event_type == KEYBOARD_EVENT)
		{
			switch(event)
			{
				case SCODEESC:
					sprite->erase(screen);
					done = 1;
					break;
				case SCODEKMINUS: case SCODEMINUS: case SCODEUNDER:
					wait_time += WAIT_50th;
					if(wait_time > WAIT_SEC5)
						wait_time = WAIT_SEC5;
					break;
				case SCODEKPLUS: case SCODEPLUS: case SCODEEQUALS:
					wait_time -= WAIT_50th;
					if(wait_time < 0)
						wait_time = 0;
					break;
				case SCODECS:
					wait_time = DEFAULT_WAIT_TIME;
					break;
				case SCODELEFT:
					seq_num = 0;
					dx = -tempdx;
					dy = 0;
					break;
				case SCODERIGHT:
					seq_num = 1;
					dx = tempdx;
					dy = 0;
					break;
				case SCODEUP:
					seq_num = 2;
					dy = -tempdy;
					dx = 0;
					break;
				case SCODEDOWN:
					seq_num = 3;
					dx = 0;
					dy = tempdy;
					break;
				case SCODEx:
					tempdx -= 1;
					if(tempdx < 0)
						tempdx = 0;
					if(seq_num==1)
						dx = tempdx;
					else if(seq_num==0)
						dx = -tempdx;
					break;
				case SCODEX:
					tempdx += 1;
					if(seq_num==1)
						dx = tempdx;
					else if(seq_num==0)
						dx = -tempdx;
					break;
				case SCODEy:
					tempdy -= 1;
					if(tempdy<0)
						tempdy = 0;
					if(seq_num==3)
						dy = tempdy;
					else if(seq_num==2)
						dy = -tempdy;
					break;
				case SCODEY:
					tempdy += 1;
					if(seq_num==3)
						dy = tempdy;
					else if(seq_num==2)
						dy = -tempdy;
					break;
				case SCODESPACE:
					seq_num = 4;
					dx = 0;
					dy = 0;
					break;
			} // end of switch event
		} // end of if event = KEYBOARD
	} while(!done);
} // Animator::FullAnimate

