#include <dos.h>
#include <string.h>
#include "anim256.h"
#include "animfunc.h"
#include "keyboard.h"
#include "menu.h"
#include "mouse.h"
#include "pcx.h"
#include "popup.h"
#include "rtrncode.h"
#include "screen.h"
#include "sprite.h"
#include "verify.h"

extern Mouse mouse;

int PopMessage(char *Message, int Buttons, BYTE *screen, FontStruct *Font,
	int textcolor)
{
	int numlines=1;
	int i;
	int Len = strlen(Message);
	int messwidth;

	for(i=0;i<Len;i++)
		if(Message[i] == '\n')
			numlines++;

	if(numlines == 1)
	{
		messwidth = Len*Font->width + 20;
		if(messwidth < ((6*Font->width)+10) * 4)
			messwidth = ((6*Font->width)+10) * 4;
		PopUp MessagePop(320/2 - messwidth/2 - 10,
			200/2 - Font->height - 10,
			320/2 + messwidth/2 + 10,
			200/2 + Font->height + 10,
			L_GRAY, WHITE, GRAY, screen, Font,
			Message, textcolor, Buttons, 3, DEFAULT_LINES);
		return MessagePop.GetResult();
	}
	else
	{
		char **Lines;
		char *cp;
		int longest = 0;
		int y1;
		int messheight;

		Lines = new char *[numlines];
		for(i=0;i<numlines;i++)
		{
			Lines[i] = new char[Len];
			sscanf(cp, "%s", Lines[i]);
			cp += (strlen(Lines[i]) + 1);
			if(strlen(Lines[i]) > longest)
				longest = strlen(Lines[i]);
		}

		messheight = numlines*(Font->height + 3) + Font->height + 15;

		y1 = 200/2 - messheight/2;

		PopUp LongMessagePop(320/2 - (longest*Font->width)/2 - 10,
			y1,
			320/2 + (longest*Font->width)/2 + 10,
			y1 + messheight,
			L_GRAY, WHITE, GRAY, screen, Font,
			"", textcolor, Buttons, 3, DEFAULT_LINES);

		for(i=0;i<numlines;i++)
		{
			GraphString(Font,
				320/2 - (longest*Font->width)/2,
				y1 + 3 + i*Font->height + 3,
				textcolor, Lines[i], screen);
			delete Lines[i];
		}
		delete Lines;
		return LongMessagePop.GetResult();
	}
} // PopMessage

void FileOpen(BYTE *screen, BYTE *shadow, FontStruct *Font, char **filename,
	int textcolor, int errortextcolor, int entrycolor, Sprite **psprite,
	Pcx **pPcx, MenuSystem *MyMenu, PaletteGrid **MyPaletteGrid,
	SpriteGrid **MySpriteGrid, SpriteDisplayer **MySpriteDisp,
	int gridcolor, int curscolor, int *NumSprites, int *SpriteWidth,
	int *SpriteHeight)
{
	TextButton Wait(100, 95, 220, 111, L_GRAY, WHITE, GRAY, SAVE_BG,
		screen, "Please Wait...", Font, textcolor,
		CENTER_TEXT, CENTER_TEXT, DEFAULT_LINES);
	EntryBox *OpenBoxes[OPEN_ENTRIES];
	OpenBoxes[0] = new EntryBox(35, 45, L_GRAY, WHITE, GRAY,
		screen, Font, 40, 60, textcolor, entrycolor, "Filename:", 30);
	OpenBoxes[1] = new EntryBox(35, 80, L_GRAY, WHITE, GRAY,
		screen, Font, 185, 80, textcolor, entrycolor, "Sprite Width:", 2);
	OpenBoxes[2] = new EntryBox(35, 95, L_GRAY, WHITE, GRAY,
		screen, Font, 185, 95, textcolor, entrycolor, "Sprite Height:", 2);
	OpenBoxes[3] = new EntryBox(35, 110, L_GRAY, WHITE, GRAY,
		screen, Font, 185, 110, textcolor, entrycolor,
		"Number of Bitmaps:", 2);
	InfoPanel *OpenPanel = new InfoPanel(25, 25, 295, 160, L_GRAY, WHITE,
		GRAY, screen, Font, "Open File", textcolor, BUT_OKCANCEL, 3,
		OpenBoxes, OPEN_ENTRIES, DEFAULT_LINES);
	int InfoResult;
	char **InfoEntries;
	char tempfilename[MAX_FILENAME_SIZE];
	int tempSpriteWidth, tempSpriteHeight, tempNumSprites;
	int rc;
	char buffer[5];

	OpenPanel->SetEntry(0, *filename);
	sprintf(buffer, "%d", *SpriteWidth);
	OpenPanel->SetEntry(1, buffer);
	sprintf(buffer, "%d", *SpriteHeight);
	OpenPanel->SetEntry(2, buffer);
	sprintf(buffer, "%d", *NumSprites);
	OpenPanel->SetEntry(3, buffer);
	InfoResult = OpenPanel->GetResult();
	if(InfoResult==BUT_CANCEL)
	{
		delete OpenBoxes[0];
		delete OpenBoxes[1];
		delete OpenBoxes[2];
		delete OpenBoxes[3];
		return;
	}
	InfoEntries = new char *[OPEN_ENTRIES];
	OpenPanel->GetEntries(InfoEntries);
	InfoResult = VerifyOpenEntries(InfoEntries, tempfilename,
					&tempSpriteWidth, &tempSpriteHeight, &tempNumSprites);
	delete InfoEntries;
	delete OpenBoxes[0];
	delete OpenBoxes[1];
	delete OpenBoxes[2];
	delete OpenBoxes[3];
	delete OpenPanel;

	if(InfoResult > 0)
	{
		char tempbuf[50];
		sprintf(tempbuf, "Too many - Using %d", InfoResult);
		PopMessage(tempbuf, BUT_OK, screen, Font, errortextcolor);
		InfoResult = GOOD;
	}

	switch(InfoResult)
	{
		case GOOD:       // verified OK
			Wait.Show();
			Pcx *tempPcx = new Pcx(screen);
			Sprite *tempSprite = new Sprite(tempNumSprites, tempSpriteWidth,
				tempSpriteHeight, screen, shadow);
			rc = tempPcx->load(tempfilename);
			delete tempPcx;
			delete tempSprite;
			if(rc!=GOOD)
			{
				if(rc==NO_FILE_OPEN)
					PopMessage("Can't open file", BUT_OK, screen, Font,
						errortextcolor);
				else if(rc==NO_MEM)
					PopMessage("Not enough memory", BUT_OK, screen, Font,
						errortextcolor);
				return;
			}
			delete *psprite;
			delete *pPcx;
			*NumSprites = tempNumSprites;
			*SpriteWidth = tempSpriteWidth;
			*SpriteHeight = tempSpriteHeight;
			strcpy(*filename, tempfilename);
			*pPcx = new Pcx(screen);
			*psprite = new Sprite(*NumSprites, *SpriteWidth, *SpriteHeight,
				screen, shadow);
			(*pPcx)->load(*filename);

			int sx=0, sy=0;
			int i=0;
			while(i<*NumSprites)
			{
				(*psprite)->grab((*pPcx)->GetImage(), i, sx, sy);
				sx += *SpriteWidth;
				if(sx+*SpriteWidth>320)
				{
					sx = 0;
					sy += *SpriteHeight;
				}
				i++;
			}
			delete *MySpriteGrid;
			delete *MySpriteDisp;
			delete *MyPaletteGrid;
			*MyPaletteGrid = new PaletteGrid(32, 8, 158, 158,
				319, 199, 0, curscolor, (*pPcx)->GetPalette(), screen);
			*MySpriteGrid = new SpriteGrid(*SpriteWidth, *SpriteHeight, 5,
				19, 155, 195, gridcolor, curscolor,
				(*psprite)->GetSpriteImage(0), *MyPaletteGrid, screen);
			*MySpriteDisp = new SpriteDisplayer(160, 20, 314, 155,
				L_GRAY, WHITE, GRAY, screen, Font, textcolor,
				*psprite, *MySpriteGrid, DEFAULT_LINES);
			(*MySpriteGrid)->AddDisplayer(*MySpriteDisp);
			Wait.Hide();
			MyMenu->Hide();
			mouse.MaybeHide();
			cls(screen);
			(*MyPaletteGrid)->Set();
			MyMenu->Show();
			(*MyPaletteGrid)->Show();
			(*MySpriteGrid)->Show();
			(*MySpriteDisp)->Show();
			mouse.MaybeShow();
			return;
		case NO_FILE_OPEN:
			PopMessage("Can't open file", BUT_OK, screen, Font,
				errortextcolor);
			return;
		case BAD_X_DIMENSION: case BAD_Y_DIMENSION:
			PopMessage("Dimensions must be 2 - 64", BUT_OK, screen, Font,
				errortextcolor);
			return;
		case BAD_NUM_SPRITES:
			PopMessage("Number of bitmaps must be 1 - 99", BUT_OK, screen,
				Font, errortextcolor);
			return;
		case NO_MEM:
			PopMessage("Not enough memory", BUT_OK, screen, Font,
				errortextcolor);
			return;
	}
}

void FileNew(BYTE *screen, BYTE *shadow, FontStruct *Font, char **filename,
	int textcolor, int errortextcolor, int entrycolor, Sprite **psprite,
	Pcx **pPcx, MenuSystem *MyMenu, PaletteGrid **MyPaletteGrid,
	SpriteGrid **MySpriteGrid, SpriteDisplayer **MySpriteDisp,
	int gridcolor, int curscolor, int *NumSprites, int *SpriteWidth,
	int *SpriteHeight)
{
	TextButton Wait(100, 95, 220, 111, L_GRAY, WHITE, GRAY, SAVE_BG,
		screen, "Please Wait...", Font, textcolor,
		CENTER_TEXT, CENTER_TEXT, DEFAULT_LINES);
	EntryBox *NewBoxes[NEW_ENTRIES];
	NewBoxes[0] = new EntryBox(35, 70, L_GRAY, WHITE, GRAY,
		screen, Font, 185, 70, textcolor, entrycolor, "Sprite Width:", 2);
	NewBoxes[1] = new EntryBox(35, 85, L_GRAY, WHITE, GRAY,
		screen, Font, 185, 85, textcolor, entrycolor, "Sprite Height:", 2);
	NewBoxes[2] = new EntryBox(35, 100, L_GRAY, WHITE, GRAY,
		screen, Font, 185, 100, textcolor, entrycolor,
		"Number of Bitmaps:", 2);
	int InfoResult;
	char **InfoEntries;
	int tempSpriteWidth, tempSpriteHeight, tempNumSprites;
	int rc;
	InfoPanel *NewPanel = new InfoPanel(25, 40, 295, 145,
		L_GRAY, WHITE, GRAY, screen, Font, "New Sprite",
		textcolor, BUT_OKCANCEL, 3, NewBoxes, NEW_ENTRIES,
		DEFAULT_LINES);

	InfoResult = NewPanel->GetResult();
	if(InfoResult==BUT_CANCEL)
	{
		delete NewBoxes[0];
		delete NewBoxes[1];
		delete NewBoxes[2];
		return;
	}
	InfoEntries = new char *[NEW_ENTRIES];
	NewPanel->GetEntries(InfoEntries);
	InfoResult = VerifyNewEntries(InfoEntries, &tempSpriteWidth,
		&tempSpriteHeight, &tempNumSprites);
	delete InfoEntries;
	delete NewBoxes[0];
	delete NewBoxes[1];
	delete NewBoxes[2];
	delete NewPanel;

	if(InfoResult > 0)
	{
		char tempbuf[50];
		sprintf(tempbuf, "Too many - Using %d", InfoResult);
		PopMessage(tempbuf, BUT_OK, screen, Font, errortextcolor);
		InfoResult = GOOD;
	}

	switch(InfoResult)
	{
		case GOOD:       // verified OK
			Wait.Show();
			Pcx *temppPcx = new Pcx(screen);
			Sprite *temppsprite = new Sprite(tempNumSprites, tempSpriteWidth,
				tempSpriteHeight, screen, shadow);
			rc = temppPcx->New();
			delete temppsprite;
			delete temppPcx;
			if(rc != GOOD)
			{
				if(rc==NO_DEFAULT_PAL)
					PopMessage("Can't open default palette",
						BUT_OK, screen, Font, errortextcolor);
				else if(rc==NO_MEM)
					PopMessage("Not enough memory", BUT_OK,
						screen, Font, errortextcolor);
				return;
			}
			delete *psprite;
			delete *pPcx;
			*NumSprites = tempNumSprites;
			*SpriteWidth = tempSpriteWidth;
			*SpriteHeight = tempSpriteHeight;
			strcpy(*filename, "");
			*pPcx = new Pcx(screen);
			*psprite = new Sprite(*NumSprites, *SpriteWidth,
				*SpriteHeight, screen, shadow);
			(*pPcx)->New();
			int sx=0, sy=0;
			int i=0;
			while(i<*NumSprites)
			{
				(*psprite)->grab((*pPcx)->GetImage(), i, sx, sy);
				sx += *SpriteWidth;
				if(sx + (*SpriteWidth) > 320)
				{
					sx = 0;
					sy += *SpriteHeight;
				}
				i++;
			}
			delete *MySpriteGrid;
			delete *MySpriteDisp;
			delete *MyPaletteGrid;
			*MyPaletteGrid = new PaletteGrid(32, 8, 158, 158,
				319, 199, 0, curscolor, (*pPcx)->GetPalette(), screen);
			*MySpriteGrid = new SpriteGrid(*SpriteWidth,
				*SpriteHeight, 5, 19, 155, 195, gridcolor,
				curscolor, (*psprite)->GetSpriteImage(0),
				*MyPaletteGrid, screen);
			*MySpriteDisp = new SpriteDisplayer(160, 20, 314, 155,
				L_GRAY, WHITE, GRAY, screen, Font, textcolor,
				*psprite, *MySpriteGrid, DEFAULT_LINES);
			(*MySpriteGrid)->AddDisplayer(*MySpriteDisp);
			Wait.Hide();
			MyMenu->Hide();
			mouse.MaybeHide();
			cls(screen);
			(*MyPaletteGrid)->Set();
			MyMenu->Show();
			(*MyPaletteGrid)->Show();
			(*MySpriteGrid)->Show();
			(*MySpriteDisp)->Show();
			mouse.MaybeShow();
			break;
		case BAD_X_DIMENSION: case BAD_Y_DIMENSION:
			PopMessage("Dimensions must be 2 - 64", BUT_OK,
				screen, Font, errortextcolor);
			break;
		case BAD_NUM_SPRITES:
			PopMessage("Number of sprites be 2 - 99", BUT_OK,
				screen, Font, errortextcolor);
			break;
		case NO_MEM:
			PopMessage("Not enough memory", BUT_OK,
				screen, Font, errortextcolor);
			break;
	}
} // FileNew
