#include <dos.h>
#include <fcntl.h>
#include <io.h>
#include <stdlib.h>
#include <string.h>
#include "anim256.h"
#include "animmain.h"
#include "animate.h"
#include "animfunc.h"
#include "editclr.h"
#include "evntmngr.h"
#include "font.h"
#include "keyboard.h"
#include "menu.h"
#include "mouse.h"
#include "mycolors.h"
#include "pcx.h"
#include "popup.h"
#include "register.h"
#include "rtrncode.h"
#include "screen.h"
#include "scancode.h"
#include "setup.h"
#include "sprite.h"
#include "tegart.h"
#include "title.h"
#include "verify.h"

BYTE CurrColor = 0;
int event_mask = MOUSE_EVENTS + KEYBOARD_EVENTS;
Mouse mouse;

EntryBox *SaveAsBoxes[SAVEPALETTE_ENTRIES];
EntryBox *SavePaletteBoxes[SAVEPALETTE_ENTRIES];
EntryBox *OpenPaletteBoxes[OPENPALETTE_ENTRIES];
EntryBox *QuickAnimateEntries[QUICKANIM_ENTRIES];
EntryBox *FullAnimateEntries[FULLANIM_ENTRIES];
EntryBox *CopyBoxes[COPY_ENTRIES];
InfoPanel *SaveAsPanel;
InfoPanel *SavePalettePanel;
InfoPanel *OpenPalettePanel;
InfoPanel *QuickAnimPanel;
InfoPanel *FullAnimPanel;
InfoPanel *CopyPanel;

int CritError = 0;

int main()
{
	BYTE *screen;
	int oldmode;
	long i;
	char **InitHeaders;
	int event, event_type, result;
	Pcx *pPcx;
	Sprite *psprite;
	PaletteGrid *MyPaletteGrid;
	SpriteGrid *MySpriteGrid;
	SpriteDisplayer *MySpriteDisp;
	MenuSystem *MyMenu;
	int NumSprites = STARTUP_NUM_SPRITES;
	int SpriteWidth = STARTUP_SPRITE_WIDTH;
	int SpriteHeight = STARTUP_SPRITE_HEIGHT;
	int textcolor = BLUE;
	int entrycolor = GREEN;
	int gridcolor = RED;
	int curscolor = L_GRAY;
	FontStruct Font;
	int errortextcolor = RED;
	char palfilename[50];
	FILE *palfp;
	int palinfile;
	char *filename;
	char **InfoEntries;
	int inforesult;
	int rc;
	int InfoResult;
	BYTE *shadow = new BYTE[64000];
	int **Sequences;
	int NumSequences;
	int *SequenceLens;

	if(init_events() == BAD)
	{
		printf("This program requires a mouse - mouse driver not loaded.\n");
		exit(1);
	}

	filename = new char[MAX_FILENAME_SIZE];
	filename[0] = 0;

	SetUpScreen(&screen, &oldmode);
	SetUpFont(&Font);

	switch(TitleScreen(screen, shadow, &Font, errortextcolor))
	{
		case GOOD:
			copyscreen(shadow, screen);
			break;
		default:
			setgmode(oldmode);
			return 1;
	}
	switch(AnimTitleScreen(screen, shadow, &Font, errortextcolor))
	{
		case GOOD:
			cls(screen);
			copyscreen(shadow, screen);
			break;
		default:
			setgmode(oldmode);
			return 1;
	}

	SetUpInitHeaders(&InitHeaders);
	SetUpEntryBoxes(screen, &Font, textcolor, entrycolor);
	SetUpInfoPanels(screen, &Font, textcolor);

	pPcx = new Pcx(screen);
	switch(pPcx->New())
	{
		case NO_MEM:
			PopMessage("Not enough memory", BUT_OK,
				screen, &Font, errortextcolor);
			setgmode(oldmode);
			return 1;
		case NO_DEFAULT_PAL:
			PopMessage("Cannot find 'DEFAULT.PAL'", BUT_OK,
				screen, &Font, errortextcolor);
			setgmode(oldmode);
			return 1;
	}

	psprite = new Sprite(NumSprites, SpriteWidth, SpriteHeight,
		screen, shadow);
	for(i=0;i<NumSprites;i++)
		psprite->grab(pPcx->GetImage(), i, i*SpriteWidth, 0);

	MyMenu = new MenuSystem(0,0,319,17,L_GRAY,WHITE,GRAY,
		InitHeaders,screen,&Font,textcolor,DEFAULT_LINES);
	MyMenu->Show();

	MyPaletteGrid = new PaletteGrid(32, 8, 158, 158, 319, 199, 0, curscolor,
		pPcx->GetPalette(), screen);
	MyPaletteGrid->Set();
	MyPaletteGrid->Show();

	MySpriteGrid = new SpriteGrid(SpriteWidth, SpriteHeight, 5, 19, 155, 195,
		gridcolor, curscolor, psprite->GetSpriteImage(0), MyPaletteGrid,
		screen);
	MySpriteGrid->Show();

	MySpriteDisp = new SpriteDisplayer(160, 20, 314, 155,
		L_GRAY, WHITE, GRAY, screen, &Font, textcolor, psprite,
		MySpriteGrid, DEFAULT_LINES);

	MySpriteGrid->AddDisplayer(MySpriteDisp);
	MySpriteDisp->Show();

	TextButton Wait(100, 95, 220, 111, L_GRAY, WHITE, GRAY,	SAVE_BG,
		screen, "Please Wait...", &Font, textcolor,
		CENTER_TEXT, CENTER_TEXT, DEFAULT_LINES);

	// Show mouse pointer
	mouse.Show();

	do{
		getevent(event_mask, &event_type, &event);
		result = HandleEvent(*MySpriteGrid, *MyPaletteGrid, *MySpriteDisp,
			*MyMenu, event_type, event);
		if(result) switch(result)
		{
			case 101:  // File Open
				FileOpen(screen, shadow, &Font, &filename,
					textcolor, errortextcolor, entrycolor,
					&psprite, &pPcx, MyMenu, &MyPaletteGrid, &MySpriteGrid,
					&MySpriteDisp, gridcolor, curscolor,
					&NumSprites, &SpriteWidth, &SpriteHeight);
				break;
			case 102:                         // New
				FileNew(screen, shadow, &Font, &filename,
					textcolor, errortextcolor, entrycolor,
					&psprite, &pPcx, MyMenu, &MyPaletteGrid, &MySpriteGrid,
					&MySpriteDisp, gridcolor, curscolor,
					&NumSprites, &SpriteWidth, &SpriteHeight);
				break;
			case 108:
				if(result == 108)
				{
					mouse.Show();
					int rc = PopMessage("Save File?", BUT_YESNOCANCEL,
							screen, &Font, textcolor);
					if(rc == BUT_NO)
						break;
					if(rc == BUT_CANCEL)
					{
						result = 0;
						break;
					}
				}
			case 103:
				if(stricmp(filename, "ANIM256.DAT")==0)
					PopMessage("Cannot overwrite ANIM256.DAT", BUT_OK,
							screen, &Font, textcolor);
				else if(filename[0])
				{
					Wait.Show();
					int sx=0, sy=0;
					i = 0;
					psprite->GetDimensions(&SpriteWidth, &SpriteHeight);
					while(i<NumSprites)
					{
						psprite->put(i, sx, sy, pPcx->GetImage());
						sx += SpriteWidth;
						if(sx+SpriteWidth >= 320)
						{
							sx = 0;
							sy += SpriteHeight;
						}
						i++;
					}
					rc = pPcx->save(filename);
					if(rc == NO_FILE_OPEN)
						PopMessage("Can't open file", BUT_OK,
							screen, &Font, errortextcolor);
					else if(rc == NO_MEM)
						PopMessage("Not enough memory", BUT_OK,
							screen, &Font, errortextcolor);
					Wait.Hide();
					break;
				}  // else there is no file name so ...
			case 104:                                 // SAVE AS
				mouse.Show();
				InfoResult = SaveAsPanel->GetResult();
				if(InfoResult==BUT_CANCEL)
				{
					if(result == 108)
					  result = 0;
					break;
				}
				InfoEntries = new char *[SAVEAS_ENTRIES];
				SaveAsPanel->GetEntries(InfoEntries);
				if(stricmp(InfoEntries[0], "ANIM256.DAT")==0)
				{
					delete InfoEntries;
					PopMessage("Cannot overwrite ANIM256.DAT", BUT_OK,
							screen, &Font, textcolor);
					break;
				}
				inforesult = VerifySaveAsEntries(InfoEntries, filename);
				delete InfoEntries;
				switch(inforesult)
				{
					case FILE_ALREADY_EXISTS: // Already exists - confirm
						int existsresult;
						mouse.Show();
						existsresult = PopMessage("File exists - Overwrite?",
							BUT_YESNO, screen, &Font, errortextcolor);
						if(existsresult == BUT_NO)
						{
							if(result == 108)
							  result = 0;
							break;
						}
					case GOOD:  // verified OK
						mouse.MaybeHide();
						Wait.Show();
						int sx=0, sy=0;
						i = 0;
						psprite->GetDimensions(&SpriteWidth, &SpriteHeight);
						while(i<NumSprites)
						{
							psprite->put(i, sx, sy, pPcx->GetImage());
							sx += SpriteWidth;
							if(sx+SpriteWidth >= 320)
							{
								sx = 0;
								sy += SpriteHeight;
							}
							i++;
						}
						rc = pPcx->save(filename);
						if(rc == NO_FILE_OPEN)
							PopMessage("Can't open file", BUT_OK,
								screen, &Font, errortextcolor);
						else if(rc == NO_MEM)
							PopMessage("Not enough memory", BUT_OK,
								screen, &Font, errortextcolor);
						Wait.Hide();
						mouse.MaybeShow();
						break;
				}
				mouse.Show();
				break;
			case 105:
				InfoResult = OpenPalettePanel->GetResult();
				if(InfoResult==BUT_CANCEL)
					break;
				InfoEntries = new char *[OPENPALETTE_ENTRIES];
				OpenPalettePanel->GetEntries(InfoEntries);
				inforesult = VerifyOpenPaletteEntries(InfoEntries,
					palfilename);
				delete InfoEntries;
				mouse.MaybeHide();
				switch(inforesult)
				{
					case GOOD:  // verified OK
						Wait.Show();
						palinfile = open(palfilename, O_BINARY);
						pPcx->load_palette(palinfile);
						close(palinfile);
						Wait.Hide();
						MyPaletteGrid->Reset(pPcx->GetPalette());
						MyPaletteGrid->Set();
						break;
					case NO_FILE_OPEN:       // bad filename
						mouse.MaybeShow();
						PopMessage("Can't open file", BUT_OK,
							screen, &Font, errortextcolor);
						break;
				}
				mouse.MaybeShow();
				break;
			case 106: case 107:
				InfoResult = SavePalettePanel->GetResult();
				if(InfoResult==BUT_CANCEL)
					break;
				InfoEntries = new char *[SAVEPALETTE_ENTRIES];
				SavePalettePanel->GetEntries(InfoEntries);
				inforesult = VerifySavePaletteEntries(InfoEntries,
					palfilename);
				char *tempfile = new char[100];
				strcpy(tempfile, InfoEntries[0]);
				delete InfoEntries;
				if(stricmp(palfilename, "DEFAULT.PAL")==0)
				{
					PopMessage("Cannot overwrite DEFAULT.PAL", BUT_OK,
							screen, &Font, textcolor);
					break;
				}
				switch(inforesult)
				{
					case FILE_ALREADY_EXISTS: // Already exists - confirm
						int existsresult;
						mouse.Show();
						existsresult = PopMessage("File exists - Overwrite?",
							BUT_YESNO, screen, &Font, errortextcolor);
						if(existsresult == BUT_NO)
						{
							delete tempfile;
							break;
						}
						strcpy(palfilename, tempfile);
						delete tempfile;
					case GOOD:  // verified OK
						mouse.MaybeHide();
						Wait.Show();
						palfp = fopen(palfilename, "wb");
						pPcx->save_palette(palfp);
						fclose(palfp);
						Wait.Hide();
						mouse.MaybeShow();
						break;
				}
				mouse.Show();
				break;
			case 201:  // Sprite Copy
				int destsprite, sourcesprite;
				InfoResult = CopyPanel->GetResult();
				if(InfoResult==BUT_CANCEL)
					break;
				InfoEntries = new char *[COPY_ENTRIES];
				CopyPanel->GetEntries(InfoEntries);
				inforesult = VerifyCopyEntries(InfoEntries, &destsprite,
					&sourcesprite, psprite);
				delete InfoEntries;
				switch(inforesult)
				{
					case GOOD:  // verified OK
						psprite->Copy(destsprite, sourcesprite);
						MySpriteDisp->Show();
						if(MySpriteDisp->GetActiveSprite() == destsprite)
							MySpriteGrid->FillGrid(
								psprite->GetSpriteImage(destsprite));
						break;
					default:              // Bad sprite number entered
						PopMessage("Bad sprite number", BUT_OK,
							screen, &Font, errortextcolor);
						break;
				}
				mouse.Show();
				break;
			case 202:  // Sprite Clear
				MyMenu->HidePullDowns();
				MySpriteGrid->Clear();
				MySpriteDisp->RefreshActive();
				break;
			case 203:  // Sprite Fill Color
				MyMenu->HidePullDowns();
				MySpriteGrid->ColorFill(CurrColor);
				MySpriteDisp->RefreshActive();
				break;
			case 204:  // Sprite Scroll Left
				MyMenu->HidePullDowns();
				MySpriteGrid->ScrollLeft();
				MySpriteDisp->RefreshActive();
				break;
			case 205:  // Sprite Scroll Right
				MyMenu->HidePullDowns();
				MySpriteGrid->ScrollRight();
				MySpriteDisp->RefreshActive();
				break;
			case 206:  // Sprite Scroll Up
				MyMenu->HidePullDowns();
				MySpriteGrid->ScrollUp();
				MySpriteDisp->RefreshActive();
				break;
			case 207:  // Sprite Scroll Down
				MyMenu->HidePullDowns();
				MySpriteGrid->ScrollDown();
				MySpriteDisp->RefreshActive();
				break;
			case 208:  // Sprite Flip Horizontal
				MyMenu->HidePullDowns();
				MySpriteGrid->HFlip();
				MySpriteDisp->RefreshActive();
				break;
			case 209:  // Sprite Flip Vertical
				MyMenu->HidePullDowns();
				MySpriteGrid->VFlip();
				MySpriteDisp->RefreshActive();
				break;
			case 210:  // Sprite Invert
				MyMenu->HidePullDowns();
				MySpriteGrid->Invert();
				MySpriteDisp->RefreshActive();
				break;
			case 301:
				ColorEditor coloreditor(25, 25, 295, 180,
					L_GRAY, WHITE, GRAY, screen, &Font,
					"Edit Color", textcolor, 3, pPcx->GetPalette(),
					CurrColor, DEFAULT_LINES);
				coloreditor.GetResult();
				break;
			case 401:
				InfoResult = QuickAnimPanel->GetResult();
				if(InfoResult==BUT_CANCEL)
					break;
				InfoEntries = new char *[QUICKANIM_ENTRIES];
				QuickAnimPanel->GetEntries(InfoEntries);
				inforesult = VerifyQuickAnimEntries(InfoEntries,
					psprite, Sequences, NumSequences, SequenceLens);
				delete InfoEntries;
				switch(inforesult)
				{
					case GOOD:  // verified OK
						Animator animator(psprite, Sequences,
							NumSequences, SequenceLens, NULL, screen);
						mouse.Hide();
						TextButton QuickBg(70, 40, 249, 189,
							L_GRAY, WHITE, GRAY, SAVE_BG, screen,
							"Quick Animate", &Font, textcolor,
							CENTER_TEXT, TOP_TEXT, DEFAULT_LINES);
						QuickBg.Show();
						GraphString(&Font, 85, 145, textcolor,
							"+ increases speed", screen);
						GraphString(&Font, 85, 155, textcolor,
							"- decreases speed", screen);
						GraphString(&Font, 85, 165, textcolor,
							"CTRL-S resets speed", screen);
						GraphString(&Font, 85, 175, textcolor,
							"ESC quits", screen);
						copyscreen(shadow, screen);
						psprite->SetNewShadow(shadow);
						animator.QuickAnimate();
						QuickBg.Hide();
						mouse.Show();
						for(i=0;i<NumSequences;i++)
							delete Sequences[i];
						delete Sequences;
						delete SequenceLens;
						break;
					default:          // Bad sequence number(s)
						PopMessage("Bad sequence", BUT_OK,
							screen, &Font, errortextcolor);
						break;
				}
				break;
			case 402:
				InfoResult = FullAnimPanel->GetResult();
				if(InfoResult==BUT_CANCEL)
					break;
				InfoEntries = new char *[FULLANIM_ENTRIES];
				FullAnimPanel->GetEntries(InfoEntries);
				inforesult = VerifyFullAnimEntries(InfoEntries, psprite,
					Sequences, NumSequences, SequenceLens);
				delete InfoEntries;
				switch(inforesult)
				{
					case GOOD:  // verified OK
						int KeyPresses[5];
						KeyPresses[0] = SCODELEFT;
						KeyPresses[1] = SCODERIGHT;
						KeyPresses[2] = SCODEUP;
						KeyPresses[3] = SCODEDOWN;
						KeyPresses[4] = SCODESPACE;
						Animator animator(psprite, Sequences,
							NumSequences, SequenceLens, KeyPresses,
							screen);
						mouse.Hide();
						TextButton FullBg(30, 40, 290, 189,
							L_GRAY, WHITE, GRAY, SAVE_BG, screen,
							"Full Animate", &Font, textcolor,
							CENTER_TEXT, TOP_TEXT, DEFAULT_LINES);
						FullBg.Show();
						GraphString(&Font, 40, 105, textcolor,
							"Arrows move", screen);
						if(SequenceLens[4])
							GraphString(&Font, 40, 115, textcolor,
								"Space stops", screen);
						if((SequenceLens[0])||(SequenceLens[1]))
							GraphString(&Font, 40, 125, textcolor,
								"x/X decreases/increases X step",
								screen);
						if((SequenceLens[2])||(SequenceLens[3]))
							GraphString(&Font, 40, 135, textcolor,
								"y/Y decreases/increases Y step",
								screen);
						GraphString(&Font, 40, 145, textcolor,
							"+ increases speed", screen);
						GraphString(&Font, 40, 155, textcolor,
							"- decreases speed", screen);
						GraphString(&Font, 40, 165, textcolor,
							"CTRL-S resets speed", screen);
						GraphString(&Font, 40, 175, textcolor,
							"ESC quits", screen);
						copyscreen(shadow, screen);
						psprite->SetNewShadow(shadow);
						animator.FullAnimate();
						FullBg.Hide();
						mouse.Show();
						for(i=0;i<NumSequences;i++)
							delete Sequences[i];
						delete Sequences;
						delete SequenceLens;
						break;
					default:               // Bad sequence number(s)
						PopMessage("Bad sequence", BUT_OK,
							screen, &Font, errortextcolor);
					break;
				}
				break;
			case 501:
				MySpriteGrid->GridOn();
				break;
			case 502:
				MySpriteGrid->GridOff();
				break;
			case 503: // View
				int sx=0, sy=0;
				i = 0;
				psprite->GetDimensions(&SpriteWidth, &SpriteHeight);
				while(i<NumSprites)
				{
					psprite->put(i, sx, sy, pPcx->GetImage());
					sx += SpriteWidth;
					if(sx+SpriteWidth > 320)
					{
						sx = 0;
						sy += SpriteHeight;
					}
					i++;
				}
				mouse.Hide();
				if(pPcx->Show()==NO_MEM)
				{
					PopMessage("Not enough memory", BUT_OK,
						screen, &Font, errortextcolor);
					break;
				}
				do {
					getevent(event_mask, &event_type, &event);
					if( (event_type == KEYBOARD_EVENT)
						|| ((event_type == MOUSE_EVENT)
						&& (event == LBUTTONUP))
						|| ((event_type == MOUSE_EVENT)
						&& (event == RBUTTONUP)) )
					{
						break;
					}
				} while(1);
				pPcx->Hide();
				mouse.Show();
				break;
			default:
				break;
		}
	} while(result != 108);

	regscreen();
	if(oldmode != 3)
	{
		do{
			getevent(event_mask, &event_type, &event);
			if((event_type == KEYBOARD_EVENT)
				|| ((event_type == MOUSE_EVENT)
					&& ((event & LBUTTONDOWN) || (event & RBUTTONDOWN))))
				break;
		}while(1);
		setgmode(oldmode);
	}
	return 0;
}; // main
