/*************************************************************************

	FILE:       	BUTTON.CPP

	AUTHOR:     	Brian Tegart, C.E.T.

	DATE:       	May 1, 1995

	DESCRIPTION:    Code for Button class

	COPYRIGHT (c) 1995 Tegart Software

==========================================================================

	Revision:	0.0

	Date:		May 1, 1995

	By:		Brian Tegart, C.E.T.

	Action:		Creation and modification from old file

**************************************************************************/

/************/
/* Includes */
/************/
#include <string.h>
#include "button.h"
#include "mouse.h"

/***********/
/* Defines */
/***********/

/********************/
/* Global variables */
/********************/

/**********************/
/* External variables */
/**********************/
extern Mouse mouse;

/******************************/
/* Static function prototypes */
/******************************/

/********************/
/* Static functions */
/********************/

/********************/
/* Public functions */
/********************/

/*------------------------------------------------------------------------

Prototype:	Button::Button(int InitX1, int InitY1, int InitX2,
			int InitY2, int InitColor, int InitTopColor,
			int InitBottomColor, int InitValue,
			BYTE *InitScreen, int InitNumEdgeLines);

Description:	Button Constructor

Input:		InitX1		- X1 Coordinate
		InitY1		- Y1 Coordinate
		InitX2		- X2 Coordinate
		InitY2		- Y2 Coordinate
		InitColor	- Face color
		InitTopColor	- Top edge color
		InitBottomColor	- Bottom edge color
		InitValue	- Value associated with button
		InitScreen	- Screen buffer for button
		InitNumEdgeLines- Thickness of edge

Modified:	none

Output:		none

Returns:	none

Side effects:	none

-------------------------------------------------------------------------*/
Button::Button(int InitX1, int InitY1, int InitX2, int InitY2,
		int InitColor, int InitTopColor, int InitBottomColor,
		int InitValue, BYTE *InitScreen, int InitNumEdgeLines)
	: Control(InitX1, InitY1, InitX2, InitY2, InitColor, InitTopColor,
		InitBottomColor, InitValue, InitScreen, InitNumEdgeLines)
{
} // Button Constructor

/*------------------------------------------------------------------------

Prototype:	Button::Push()

Description:	Pushes the button (Swaps the edge colors)

Input:		none

Modified:	none

Output:		none

Returns:	none

Side effects:	none

-------------------------------------------------------------------------*/
void Button::Push()
{
	if((Active) || (!Visible))
		return;
	Box::SwapEdgeColors();
	Activate();
} // Button::Push

/*------------------------------------------------------------------------

Prototype:	Button::Release()

Description:	Releases the button (Swaps the edge colors)

Input:		none

Modified:	none

Output:		none

Returns:	none

Side effects:	none

-------------------------------------------------------------------------*/
void Button::Release()
{
	if((!Active) || (!Visible))
		return;
	Box::SwapEdgeColors();
	Deactivate();
} // Button::Release

/*------------------------------------------------------------------------

Prototype:	TextButton::TextButton(int InitX1, int InitY1, int InitX2,
			int InitY2, int InitColor, int InitTopColor,
			int InitBottomColor, int InitValue, char *InitText,
			int InitTextColor, FontStruct *InitFontPtr,
			BYTE *InitScreen, int InitXJust, int InitYJust,
			int InitNumEdgeLines)

Description:	TextButton Constructor

Input:		InitX1		- X1 Coordinate
		InitY1		- Y1 Coordinate
		InitX2		- X2 Coordinate
		InitY2		- Y2 Coordinate
		InitColor	- Face color
		InitTopColor	- Top edge color
		InitBottomColor	- Bottom edge color
		InitValue	- Value associated with button
		InitText	- Pointer to text to display on button
		InitTextColor	- Text color
		InitFontPtr	- Pointer to font info
		InitScreen	- Screen buffer for button
		InitXJust	- X justification
		InitYJust	- Y justification
		InitNumEdgeLines- Thickness of edge

Modified:	none

Output:		none

Returns:	none

Side effects:	none

Notes:		if X2 and/or Y2 is AUTO_CALC, X2 and/or Y2 is automatically
		calculated to 'shrink wrap' the text, and InitXJust and/or
		InitYJust is ignored (changed to CENTER_TEXT)

-------------------------------------------------------------------------*/
// TextButton Constructor
TextButton::TextButton(int InitX1, int InitY1, int InitX2, int InitY2,
			int InitColor, int InitTopColor, int InitBottomColor,
			int InitValue, char *InitText, int InitTextColor,
			FontStruct *InitFontPtr, BYTE *InitScreen,
			int InitXJust, int InitYJust, int InitNumEdgeLines)
	: Button(InitX1, InitY1, InitX2, InitY2, InitColor, InitTopColor,
		InitBottomColor, InitValue, InitScreen, InitNumEdgeLines)
{
	TextColor = InitTextColor;
	FontPtr = InitFontPtr;
	Text = InitText;
	if(InitX2 == AUTO_CALC)
	{
		X_Just = CENTER_TEXT;
		X2 = X1 + (FontPtr->width * (strlen(Text)+1));
	}
	else
		X_Just = InitXJust;
	if(InitY2 == AUTO_CALC)
	{
		Y_Just = CENTER_TEXT;
		Y2 = Y1 + (FontPtr->height * 2);
	}
	else
		Y_Just = InitYJust;
} // TextButton Constructor

void TextButton::Show()
{
	int XPos, YPos;
	int Len = strlen(Text);
	int Lines = NumEdgeLines;

	if(Visible)
		return;

	Button::Show();
	if(Text[0])
	{
		switch(X_Just)
		{
			case LEFT_TEXT:
				XPos = X1 + Lines + 3;
				break;
			case RIGHT_TEXT:
				XPos = X2 - Lines - 3 - (strlen(Text)*FontPtr->width);
				break;
			default:  // case CENTER_TEXT:
				XPos = ((X1+X2)/2)-(Len*(FontPtr->width / 2));
				break;
		}
		switch(Y_Just)
		{
			case TOP_TEXT:
				YPos = Y1 + Lines + 3;
				break;
			case BOTTOM_TEXT:
				YPos = Y2 - Lines - 3 - FontPtr->height;
				break;
			default:  // case CENTER_TEXT:
				YPos = ((Y1+Y2)/2) - (FontPtr->height / 2) + 1;
				break;
		}
		mouse.MaybeHide();
		GraphString(FontPtr, XPos, YPos, TextColor, Text, screen);
		mouse.MaybeShow();
	}
} // TextButton::Show

void TextButton::Hide()
{
	if(!Visible)
		return;
	Button::Hide();
} // TextButton::Hide
