#include <stdio.h>
#include <stdlib.h>
#include "evntmngr.h"
#include "joystick.h"
#include "keyboard.h"
#include "mouse.h"
#include "rtrncode.h"
#include "scancode.h"

// Variable declarations
int xmin,xmax,xcent,ymin,ymax,ycent;    // Joystick calibration variables
extern BYTE CurrColor;
extern Mouse mouse;

// Initialize event manager
int init_events()
{
	// Initialize the mouse driver
	if(mouse.IsInitialized() == FALSE)
		return BAD;
	return GOOD;
}

// Set minimum joystick coordinates
void setmin()
{
	while (!readjbutton(JBUTTON1)); // Loop until joystick button pressed
	xmin=readstick(JOY_X);          // Get x coordinate
	ymin=readstick(JOY_Y);          // Get y coordinate
	while (readjbutton(JBUTTON1));  // Loop until button released
}

// Set maximum joystick coordinates
void setmax()
{
	while (!readjbutton(JBUTTON1)); // Loop until joystick button pressed
	xmax=readstick(JOY_X);          // Get x coordinate
	ymax=readstick(JOY_Y);          // Get y coordinate
	while (readjbutton(JBUTTON1));  // Loop until button released
}

// Set center joystick coordinates
void setcenter()
{
	while (!readjbutton(JBUTTON1)); // Loop until joystick button pressed
	xcent=readstick(JOY_X);         // Get x coordinate
	ycent=readstick(JOY_Y);         // Get y coordinate
	while (readjbutton(JBUTTON1));  // Loop until button released
}

// Get events from devices selected by EVENT_MASK
void getevent(int event_mask, int *event_type, int *event)
{
	int event_return=NOEVENT;       // Initialize events to NO EVENTS
	static int lastlbutton=0, lastrbutton=0;
	static int x,y;             // Relative mouse coordinates

	*event_type = NOEVENT;
	// If joystick events requested....
	if (event_mask & JOYSTICK_EVENTS)
	{
		// ...set left, right, up, down and button events:
		if(readstick(JOY_X)<(xcent-4))
			event_return|=LEFT;
		if(readstick(JOY_X)>(xcent+10))
			event_return|=RIGHT;
		if(readstick(JOY_Y)<(xcent-4))
			event_return|=UP;
		if(readstick(JOY_Y)>(xcent+10))
			event_return|=DOWN;
		if(readjbutton(JBUTTON1))
			event_return|=LBUTTONDOWN;
		if(readjbutton(JBUTTON2))
			event_return|=RBUTTONDOWN;
	}
	if(event_return)
		*event_type |= JOYSTICK_EVENT;
	// If mouse events requested....
	if(event_mask & MOUSE_EVENTS)
	{
		// ...set left, right, up, down and button events:
		mouse.GetRelPos(&x,&y);	// Read relative mouse position
		if(x<0)
		{
			event_return |= LEFT;
			*event_type |= MOUSE_EVENT;
		}
		if(x>0)
		{
			event_return |= RIGHT;
			*event_type |= MOUSE_EVENT;
		}
		if(y<0)
		{
			event_return |= UP;
			*event_type |= MOUSE_EVENT;
		}
		if(y>0)
		{
			event_return |= DOWN;
			*event_type |= MOUSE_EVENT;
		}
		int b=mouse.ReadButtons();             // Read mouse button
		if(b&MLEFTBUTTON_DOWN)
		{
			if(lastlbutton)
				event_return |= LBUTTONHELD;
			else
			{
				event_return |= LBUTTONDOWN;
				lastlbutton = 1;
			}
			*event_type |= MOUSE_EVENT;
		}
		else if(lastlbutton)
		{
			event_return |= LBUTTONUP;
			lastlbutton = 0;
			*event_type |= MOUSE_EVENT;
		}
		if(b&MRIGHTBUTTON_DOWN)
		{
			if(lastrbutton)
				event_return |= RBUTTONHELD;
			else
			{
				event_return |= RBUTTONDOWN;
				lastrbutton = 1;
			}
			*event_type |= MOUSE_EVENT;
		}
		else if(lastrbutton)
		{
			event_return |= RBUTTONUP;
			lastrbutton = 0;
			*event_type |= MOUSE_EVENT;
		}
	}
	// If keyboard events requested
	if(event_mask & KEYBOARD_EVENTS)
	{
		int k=scankey();            // Read scan code of last key pressed
		// ...and determine which key event, if any, occurred:
		if(k)
		{
			event_return = k; // keyboard overrules mouse and joystick
			*event_type = KEYBOARD_EVENT;
		}
	}
	*event = event_return;
} // getevent

int MouseWithin(SpriteGrid &MySpriteGrid)
{
	int sx1,sx2,sy1,sy2,mx,my;

	MySpriteGrid.GetCoords(&sx1, &sy1, &sx2, &sy2);
	mouse.GetPos(&mx, &my);
	if((mx>sx1) && (mx<sx2) && (my>sy1) && (my<sy2))
		return TRUE;
	return FALSE;
} // MouseWithin (overloaded for SpriteGrid)

int MouseWithin(PaletteGrid &MyPaletteGrid)
{
	int px1,px2,py1,py2,mx,my;

	MyPaletteGrid.GetCoords(&px1, &py1, &px2, &py2);
	mouse.GetPos(&mx, &my);
	if((mx>px1) && (mx<px2) && (my>py1) && (my<py2))
		return TRUE;
	return FALSE;
} // MouseWithin (overloaded for PaletteGrid)

int MouseWithin(SpriteDisplayer &MySpriteDisp)
{
	int px1,px2,py1,py2,mx,my;

	MySpriteDisp.GetCoords(&px1, &py1, &px2, &py2);
	mouse.GetPos(&mx, &my);
	if((mx>px1) && (mx<px2) && (my>py1) && (my<py2))
		return TRUE;
	return FALSE;
} // MouseWithin (overloaded for SpriteDisplayer)

int MouseWithin(MenuSystem &MyMenu)
{
	int px1,px2,py1,py2,mx,my;

	MyMenu.GetCoords(&px1, &py1, &px2, &py2);
	mouse.GetPos(&mx, &my);
	if( (mx>px1) && (mx<px2) && (my>py1) && (my<py2) )
		return TRUE;
	if(MyMenu.AnyActive())
		return TRUE;
	return FALSE;
} // MouseWithin (overloaded for MenuSystem)

int HandleEvent(SpriteGrid &MySpriteGrid, PaletteGrid &MyPaletteGrid,
		SpriteDisplayer &MySpriteDisp, MenuSystem &MyMenu,
		int EventType, int Event)
{
	int SelectedPullDown, SelectedTopic;
	if(EventType != KEYBOARD_EVENT)
	{
		if((MouseWithin(MySpriteGrid)) && (!MouseWithin(MyMenu)))
			mouse.Hide();
		else
			mouse.Show();
		if(MouseWithin(MyMenu))
		{
			MyMenu.HandleEvent(EventType, Event,
				&SelectedPullDown, &SelectedTopic);
			return(SelectedPullDown*100+SelectedTopic);
		}
		if(MouseWithin(MySpriteGrid))
			MySpriteGrid.HandleEvent(EventType, Event);
		else if(MouseWithin(MyPaletteGrid))
			MyPaletteGrid.HandleEvent(EventType, Event);
		else if(MouseWithin(MySpriteDisp))
			MySpriteDisp.HandleEvent(EventType, Event);
	}
	else
	{
		if(MyMenu.AnyActive())
		{
			MyMenu.HandleEvent(EventType, Event,
				&SelectedPullDown, &SelectedTopic);
			return(SelectedPullDown*100+SelectedTopic);
		}
		else
		{
			switch(Event)
			{
				case SCODEAF: case SCODEAB: case SCODEAC:
				case SCODEAA: case SCODEAM:
					MyMenu.HandleEvent(EventType, Event,
					&SelectedPullDown, &SelectedTopic);
					return(SelectedPullDown*100+SelectedTopic);
				case SCODEAX:
					return 108;
				case SCODEUP: case SCODEDOWN: case SCODELEFT:
				case SCODERIGHT: case SCODEENTER: case SCODEESC:
				case SCODESPACE:
					if(MouseWithin(MySpriteGrid))
						MySpriteGrid.HandleEvent(EventType, Event);
					else if (MouseWithin(MyPaletteGrid))
						MyPaletteGrid.HandleEvent(EventType, Event);
					break;
				case SCODEPGUP: case SCODEPGDN:
					MySpriteDisp.HandleEvent(EventType, Event);
					break;
				case SCODECR:
					MySpriteGrid.ReplaceColor(CurrColor);
					break;
			} // end of switch
		} // end of else (no menu active)
	} // end of else (event is a keyboard event)
	return 0;
} // HandleEvent

