;*************************************************************************
;
;	FILE:		FONT.ASM
;
;	AUTHOR:		Brian Tegart, C.E.T.
;
;	DATE:		April 28, 1995
;
;	DESCRIPTION:	Bitmapped Font routines
;
;	COPYRIGHT (c) 1995 Tegart Software
;
;=========================================================================
;
;	Revision:	0.0
;
;	Date:		April 28, 1995
;
;	By:		Brian Tegart, C.E.T.
;
;	Action:		Creation and modificiation from old code
;
;*************************************************************************

;**************
;* Directives *
;**************

.286
.MODEL        large
.CODE

;************
;* Includes *
;************

;*************************
;* Function declarations *
;*************************

PUBLIC        _G8x8FontAddr, _GetFontAddr, _GraphString

;***********
;* Equates *
;***********

SCREEN_WIDTH	equ	320
SCREEN_HEIGHT	equ	200

;**************
;* Structures *
;**************

FontStruct STRUC
	FontWidth			DW	?
	FontHeight			DW	?
	FontBytesPerChar		DB	?
	FontPtr				DD	?
FontStruct ENDS

;***************
;* Global data *
;***************

;*****************
;* External data *
;*****************

;********************
;* Public functions *
;********************

;------------------------------------------------------------------------
;
;Prototype:	void G8x8FontAddr(WORD *FontSeg, WORD *FontOffset);
;
;Description:   Get address of ROM 8x8 font
;
;Input:		none
;
;Modified:	none
;
;Output:	FontSeg		- pointer to hold Segment of ROM 8x8 font
;		FontOffset	- pointer to hold Offset of ROM 8x8 font
;
;Returns:	none
;
;Side effects:	none
;
;------------------------------------------------------------------------
_G8x8FontAddr   PROC
	ARG	FontSeg:DWORD,FontOffset:DWORD
	push	bp
	mov	bp,sp
	push	ax
	push	bx
	push	cx
	push	dx
	push	es
	mov	ax,1130h		; Function 11h, subfunc 30h
	mov	bh,03h			;  (Get Font info - 00-7f characters)
	push	bp
	int	10h			; Call video interrupt
	mov	cx,es			; Save segment of font buffer
	mov	dx,bp			; Save offset of font buffer
	pop	bp
	les	bx,FontSeg		; Point es:bx at FontSeg parameter
	mov	[es:bx],cx		; ...and store segment
	les	bx,FontOffset		; Point es:bx at FontOffet parameter
	mov	[es:bx],dx		; ...and store offset
	pop	es
	pop	dx
	pop	cx
	pop	bx
	pop	ax
	mov	sp,bp
	pop	bp
	ret
_G8x8FontAddr   ENDP

;------------------------------------------------------------------------
;
;Prototype:	void GetFontAddr(int fonttype, WORD *fontseg, WORD *fontoff,
;			int *bytesperchar);
;
;Description:	Gets address of font
;
;Input:		fonttype	- type of font (see FONT.H for defines)
;
;Modified:	none
;
;Output:	fontseg		- pointer to hold segment of font
;		fontoff		- pointer to hold offset of font
;		bpc		- bytes per character
;
;Returns:	none
;
;Side effects:	none
;
;------------------------------------------------------------------------
_GetFontAddr   PROC
	ARG	fonttype:WORD,fontseg:DWORD,fontoff:DWORD,bpc:DWORD
	push	bp
	mov	bp,sp
	push	es
	mov	ax,1130h		; Function 11h, subfunc 30h
					;  (get font info)
	mov	bx,fonttype
	mov	bh,bl
	push	bp
	int	10h			; Call video interrupt
	mov	ax,cx			; Save bytes per character
	mov	cx,es			; Save segment of font buffer
	mov	dx,bp			; Save offset of font buffer
	pop	bp
	les	bx,fontseg		; Point es:bx at fontseg parameter
	mov	[es:bx],cx		; ...and store segment
	les	bx,fontoff		; Point es:bx at fontoff parameter
	mov	[es:bx],dx		; ...and store offset
	les	bx,bpc			; Point es:bx at bpc parameter
	mov	[es:bx],al		; ...and store bytes per character
	pop	es
	mov	sp,bp
	pop	bp
	ret
_GetFontAddr   ENDP

;------------------------------------------------------------------------
;
;Prototype:	void GraphString(FontStruct *Font, int x, int y,
;			char clr, char *strng, BYTE *screen);
;
;Description:	Prints a string in 320x200 mode (mode 0x13)
;
;Input:		Font		- pointer to font info
;		x		- x coordinate to start
;		y		- y coordinate to start
;		clr		- color to use
;		strng		- string to print
;		screen		- pointer to screen buffer
;
;Modified:	none
;
;Output:	none
;
;Returns:	none
;
;Side effects:	none
;
;------------------------------------------------------------------------
_GraphString PROC
	ARG	fptr:DWORD,x:WORD,y:WORD,clr:BYTE,strng:DWORD,screen:DWORD
	LOCAL	fheight:WORD,fwidth:WORD,bpc:WORD=AUTO_SIZE
	push	bp
	mov	bp,sp
	sub	sp,AUTO_SIZE
	push	ds
	push	es
	push	di
	push	si
	les	di,screen		; Point ES:DI at screen
	add	di,x
	mov	ax,y
	mov	bx,y
	shl	ax,8			; Find y * 256
	shl	bx,6			; Find y * 64
	add	ax,bx			; Add them together to get y * 320
	add	di,ax			; Add to DI to point ES:DI at coord
	lds	si,fptr			; Point DS:SI to FontStruct
	mov	ax,[si].FontHeight	; Lines per char
	mov	fheight,ax
	mov	ax,[si].FontWidth	; pixels per line
	mov	fwidth,ax
	mov	al,[si].FontBytesPerChar; Bytes per char
	xor	ah,ah
	mov	bpc,ax
	lds	si,[si].FontPtr		; Point DS:SI to font buffer
	xor	bx,bx			; 0 characters done
	xor	dh,dh
NEXTCHAR:
	push	ds			; Save pointer to font buffer
	push	si
	lds	si,strng		; Point DS:SI to string
	add	si,bx			;  and character to display
	mov	dl,[ds:si]		; Load DL with character
	cmp	dl,0
	je	DONE_GRAPHSTRING	; If a null, done
	pop	si			; Restore pointer to font buffer
	pop	ds
	push	si			; Save the pointer to the font buffer
	mov	ax,dx			; Get copy of character
	mov	dx,bpc
	xor	dh,dh
	mul	dl			; Multiply al by bpc
	add	si,ax			; Find offset into font buffer
	mov	al,clr			; Color to display
	mov	cx,fheight		; lines per char
NEXTLINE:
	push	cx
	mov	cx,fwidth		; Pixels per line
	mov	ah,[ds:si]		; Load font info byte
NEXTPIXINLINE:
	shl	ah,1			; Shift out MSB into carry
	jnc	NOTTHISPIXEL		; If not a 1, do not put pixel
	mov	[es:di],al		; Was a 1, put a pixel
NOTTHISPIXEL:
	inc	di			; Point to next pixel
	loop	NEXTPIXINLINE		; Do next pixel
	add	di,SCREEN_WIDTH		; Move to next line of character
	sub	di,fwidth
	pop	cx			; Restore line counter
	inc	si			; Inc font pointer
	loop	NEXTLINE		; Loop for entire character
	inc	bx			; Keep track of characters displayed
	mov	ax,fheight
	mov	dx,SCREEN_WIDTH
	mul	dx
	sub	ax,fwidth
	sub	di,ax			; Move Pointer to next coord
	pop	si			; Point font pointer to beginning
	jmp	NEXTCHAR		; Next character
DONE_GRAPHSTRING:
	pop	si
	pop	ds
	pop	si
	pop	di
	pop	es
	pop	ds
	mov	sp,bp
	pop	bp
	ret
_GraphString ENDP

	END

