/*************************************************************************

	FILE:		GRID.CPP

	AUTHOR:		Brian Tegart, C.E.T.

	DATE:		April 30, 1995

	DESCRIPTION:	Code for Grid class

	COPYRIGHT (c) 1995 Tegart Software

==========================================================================

	Revision:	0.0

	Date:		April 30, 1995

	By:		Brian Tegart, C.E.T.

	Action:		Creation and modification from old file

**************************************************************************/

/************/
/* Includes */
/************/
#include "grid.h"

/***********/
/* Defines */
/***********/

/********************/
/* Global variables */
/********************/

/**********************/
/* External variables */
/**********************/

/******************************/
/* Static function prototypes */
/******************************/

/********************/
/* Static functions */
/********************/

/********************/
/* Public functions */
/********************/

/*------------------------------------------------------------------------

Prototype:	Grid::Grid(int InitAcross, int InitDown, int InitX1,
			int InitY1, int InitX2, int InitY2, int InitColor)

Description:	Grid constructor

Input:		InitAcross	- initial sections across
		InitDown	- initial sections down
		InitX1		- initial X1 coordinate
		InitY1		- initial Y1 coordinate
		InitX2		- initial X2 coordinate
		InitY2		- initial Y2 coordinate
		InitColor	- initial color

Modified:	none

Output:		none

Returns:	none

Side effects:	none

Notes:		Initial coordinates may be changed to find a "best fit"
		for the grid - the grid will be centered within the
		passed initial coordinates

-------------------------------------------------------------------------*/
Grid::Grid(int InitAcross, int InitDown, int InitX1, int InitY1,
		int InitX2, int InitY2, int InitColor)
	: Rectangle(InitX1, InitY1, InitX2, InitY2, InitColor)
{
	int XOff, YOff;
	int ActX1, ActY1, ActX2, ActY2;

	if(InitAcross<=0)
		Across = 1;
	else
		Across = InitAcross;
	if(InitDown<=0)
		Down = 1;
	else
		Down = InitDown;

	CellWidth = (InitX2 - InitX1) / Across;
	CellHeight = (InitY2 - InitY1) / Down;

	XOff = ((InitX2-InitX1) - (CellWidth*Across)) / 2;
	YOff = ((InitY2-InitY1) - (CellHeight*Down)) / 2;

	ActX1 = InitX1 + XOff;
	ActY1 = InitY1 + YOff;
	ActX2 = ActX1 + (Across * CellWidth);
	ActY2 = ActY1 + (Down * CellHeight);
	MoveTo(ActX1, ActY1, ActX2, ActY2);
} // Grid Constructor

