;*************************************************************************
;
;	FILE:		JOYSTICK.ASM
;
;	AUTHOR:		Brian Tegart, C.E.T.
;
;	DATE:		April 30, 1995
;
;	DESCRIPTION:	Joystick routines
;
;	COPYRIGHT (c) 1995 Tegart Software
;
;	NOTES:		Some source used from the book "Flights of Fanatasy"
;
;=========================================================================
;
;	Revision:	0.0
;
;	Date:		April 30, 1995
;
;	By:		Brian Tegart, C.E.T.
;
;	Action:		Creation and modification from old file
;
;*************************************************************************

;**************
;* Directives *
;**************
.MODEL	large
.CODE

;************
;* Includes *
;************

;*************************
;* Function declarations *
;*************************
PUBLIC	_readstick,_readjbutton

;***********
;* Equates *
;***********
GAMEPORT 	equ 0201h

;**************
;* Structures *
;**************

;***************
;* Global data *
;***************

;*****************
;* External data *
;*****************

;********************
;* Public functions *
;********************

;------------------------------------------------------------------------
;
;Prototype:	int readstick(int bmask);
;
;Description:   Read current position of joystick on axis specified by bmask
;
;Input:		bmask		- bitmask for axis
;
;Modified:	none
;
;Output:	none
;
;Returns:	Joystick position
;
;Side effects:	none
;
;------------------------------------------------------------------------
_readstick	PROC
	ARG	bmask:WORD
	push	bp
	mov	bp,sp
	cli				; Turn off interrupts, which could
					;	effect timing
	mov	ah,byte ptr bmask	; Get bitmask into ah.
	xor	al,al
	mov	dx,GAMEPORT		; Point DX at joystick port
	xor	cx,cx			; Prepare to loop 65,536 times
	out	dx,al			; Set joystick bits to 1
loop2:
	in	al,dx			; Read joystick bits
	test	al,ah			; Is requested bit (in bitmask)
					;  still 1?
	loopne	loop2			; If so (and maximum count isn't
					;  done), try again
	sti				; Count is finished, so reenable
					;  interrupts
	xor	ax,ax
	sub	ax,cx			; Subtract CX from zero, to get count
	pop	bp
	ret
_readstick	ENDP

;------------------------------------------------------------------------
;
;Prototype:	int readjbutton(int bmask);
;
;Description:   Read current joystick button specified by bmask
;
;Input:		bmask		- bitmask for button
;
;Modified:	none
;
;Output:	none
;
;Returns:	0 or 1, indicating state of button
;
;Side effects:	none
;
;------------------------------------------------------------------------
_readjbutton	PROC
	ARG	bmask:WORD
	push	bp
	mov	bp,sp
	mov	dx,GAMEPORT		; Point DX at joystick port
	xor	ah,ah			; Zero high byte of return value
	out	dx,al			; Request data from port
	in	al,dx			; Get value from joystick
	not	al			; Flip button bits
	mov	bx,bmask		; Mask out all but requested buttons
	and	al,bl			; And leave result in AX
	pop	bp
	ret
_readjbutton	ENDP

	END
