;*************************************************************************
;
;	FILE:		LINE.ASM
;
;	AUTHOR:		Brian Tegart, C.E.T.
;
;	DATE:		April 28, 1995
;
;	DESCRIPTION:	Line and box drawing routines for mode 0x13
;
;	COPYRIGHT (c) 1995 Tegart Software
;
;=========================================================================
;
;	Revision:	0.0
;
;	Date:		April 28, 1995
;
;	By:		Brian Tegart, C.E.T.
;
;	Action:		Creation and modification from old file
;
;*************************************************************************

;**************
;* Directives *
;**************
.286
.MODEL	large
.CODE

;************
;* Includes *
;************

;*************************
;* Function declarations *
;*************************
PUBLIC	_linedraw, _hlinedraw, _vlinedraw, _boxdraw, _negatebox

;***********
;* Equates *
;***********
SCREEN_WIDTH	equ	320
SCREEN_HEIGHT	equ	200
SCREEN_AREA	equ	SCREEN_WIDTH*SCREEN_HEIGHT

;**************
;* Structures *
;**************

;***************
;* Global data *
;***************

;*****************
;* External data *
;*****************

;********************
;* Public functions *
;********************

;------------------------------------------------------------------------
;
;Prototype:	void hlinedraw(int X1, int Y1, int X2, int color,
;			BYTE *screen);
;
;Description:	Draw a horizontal line
;
;Input:		X1		- starting X coordinate
;		Y1		- starting Y coordinate
;		X2		- ending X coordinate
;		color		- color for line
;
;Modified:	screen		- pointer to screen
;
;Output:	none
;
;Returns:	none
;
;Side effects:	none
;
;------------------------------------------------------------------------
_hlinedraw	PROC
	ARG	x1:WORD,y1:WORD,x2:WORD,color:WORD, \
		scr_off:WORD,scr_seg:WORD
	push	bp
	mov	bp,sp
	push	di
	push	es
	push	ax
	push	bx
	push	cx
	push	dx
	mov	ax,x2			; Test to see if X2 < X1
	sub	ax,x1
	jae	drawhline
	mov	ax,x2			; X2 is less, so a swap is reqrd
	mov	bx,x1
	mov	x2,bx
	mov	x1,ax
drawhline:
	mov	ax,scr_seg		; Get screen segment in ax
	mov	es,ax
	mov	ax,y1			; Get y1 in ax...
	mov	bx,ax
	shl	ax,8			; Find y1 * 256
	shl	bx,6			; Find y1 * 64
	add	ax,bx			; Add them together to get y1 * 320
	add	ax,x1			; And add x1 to get pixel offset
	add	ax,scr_off		; Add screen offset
	mov	di,ax			; Move offset to DI
	mov	ax,color
	mov	ah,al
	cld
	mov	cx,x2
	sub	cx,x1			; Get change in x into DX
	inc	cx			; Needes to draw to last X coord
	sar	cx,1			; Divide by 2
	jnc	drawhline2		; If even number of bytes, jump
	stosb				; Wasn't, so draw first one then..
drawhline2:
	rep	stosw			; Draw the line
	pop	dx
	pop	cx
	pop	bx
	pop	ax
	pop	es
	pop	di
	mov	sp,bp			; Finished!
	pop	bp
	ret
_hlinedraw	ENDP

;------------------------------------------------------------------------
;
;Prototype:	void vlinedraw(int X1, int Y1, int Y2, int color,
;			BYTE *screen);
;
;Description:	Draw a vertical line
;
;Input:		X1		- starting X coordinate
;		Y1		- starting Y coordinate
;		Y2		- ending Y coordinate
;		color		- color for line
;
;Modified:	screen		- pointer to screen
;
;Output:	none
;
;Returns:	none
;
;Side effects:	none
;
;------------------------------------------------------------------------
_vlinedraw	PROC
	ARG	x1:WORD,y1:WORD,y2:WORD,color:WORD,\
		scr_off:WORD,scr_seg:WORD
	push	bp
	mov	bp,sp
	push	di
	push	es
	push	cx
	mov	ax,y2			; Test to see if Y2 < Y1
	sub	ax,y1
	jae	drawvline
	mov	ax,y2			; Y2 is less, so a swap is reqrd
	mov	bx,y1
	mov	y2,bx
	mov	y1,ax
drawvline:
	mov	ax,scr_seg		; Get screen segment in ax
	mov	es,ax
	mov	ax,y1			; Get y1 in ax...
	mov	bx,ax
	shl	ax,8			; Find y1 * 256
	shl	bx,6			; Find y1 * 64
	add	ax,bx			; Add them together to get y1 * 320
	add	ax,x1			; And add x1 to get pixel offset
	add	ax,scr_off		; Add screen offset
	mov	di,ax			; Move offset to DI
	mov	cx,y2
	sub	cx,y1			; Get change in x into DX
	inc	cx			; Needed to draw to last Y coord
	mov	ax,color		; Color to draw
putnextvpixel:
	stosb				; Draw the pixel
	add	di,SCREEN_WIDTH-1	; Down 1 line (stosb already incs di)
	loop	putnextvpixel
	pop	cx
	pop	es
	pop	di
	mov	sp,bp			; Finished!
	pop	bp
	ret
_vlinedraw	ENDP

;------------------------------------------------------------------------
;
;Prototype:	void linedraw(int x1, int y1, int x2, int y2, int color,
;			BYTE *screen)
;
;Description:	Draw a line
;
;Input:		x1		- starting X coordinate
;		y1		- starting Y coordinate
;		x2		- ending X coordinate
;		y2		- ending Y coordinate
;		color		- color for line
;
;Modified:	screen		- pointer to screen
;
;Output:	none
;
;Returns:	none
;
;Side effects:	none
;
;Notes:		Taken and modified from the book "Flights of Fantasy"
;
;------------------------------------------------------------------------
_linedraw	PROC
	ARG	x1:WORD,y1:WORD,x2:WORD,y2:WORD,color:WORD,\
		scr_off:WORD,scr_seg:WORD
	LOCAL	y_unit:WORD,x_unit:WORD,xdiff:WORD,ydiff:WORD,\
		error_term:WORD=AUTO_SIZE
	push	bp
	mov	bp,sp
	sub	sp,AUTO_SIZE
	mov	ax,y2
	sub	ax,y1
	jne	not_hline
	call	_hlinedraw	C,x1,y1,x2,color,scr_off,scr_seg
	jmp	linedone
not_hline:
	mov	ax,x2
	sub	ax,x1
	jne	not_vline
	call	_vlinedraw	C,x1,y1,y2,color,scr_off,scr_seg
	jmp	linedone
not_vline:
	mov	ax,scr_seg		; Get screen segment in ax
	mov	es,ax
	mov	ax,y1			; Get y1 in ax...
	mov	bx,ax
	shl	ax,8			; Find y1 * 256
	shl	bx,6			; Find y1 * 64
	add	ax,bx			; Add them together to get y1 * 320
	add	ax,x1			; And add x1 to get pixel offset
	add	ax,scr_off		; Add screen offset
	mov	bx,ax			; Move offset to BX
init_line:
	mov	dx,color		; Put pixel color in dx
	mov	error_term,0		; Initialize error term
	mov	ax,y2			; Determine sign of y2-y1
	sub	ax,y1
	jns	ypos			; If positive, jump
	mov	y_unit,-SCREEN_WIDTH	; Else handle negative slope
	neg	ax			; Get absolute value of YDIFF
	mov	ydiff,ax		; And store it in memory
	jmp	next
ypos:
	mov	y_unit,SCREEN_WIDTH	; Handle positive slope
	mov	ydiff,ax		; Store YDIFF
next:
	mov	ax,x2			; Determine sign of x2-x1
	sub	ax,x1
	jns	xpos			; If positive, jump
	mov	x_unit,-1		; Else handle negative case
	neg	ax			; Get absolute value of XDIFF
	mov	xdiff,ax		; And store it in memory
	jmp	next2
xpos:
	mov	x_unit,1		; Handle positive case
	mov	xdiff,ax		; Store XDIFF in memory
next2:
	cmp	ax,ydiff		; Compare XDIFF (in AX) and YDIFF
	jc	yline			; IF XDIFF<YDIFF then count
					;	in Y dimension
xline:					; Slope less than one,
					;	so increment in x dimension
	mov	cx,xdiff		; Get line length in cx for count
	inc	cx
xline1:
	mov	es:[bx],dl		; Draw next point in line
	add	bx,x_unit		; Point offset to next pixel in
					;	x direction
	mov	ax,error_term		; Check to see if move required
					;	in Y direction?
	add	ax,ydiff
	mov	error_term,ax
	sub	ax,xdiff
	jc	xline2			; If not, continue
	mov	error_term,ax
	add	bx,y_unit		; Else, move up or down one pixel
xline2:
	loop	xline1			; Loop until count (in CX) complete
	jmp	linedone
yline:					; Count in y dimension
	mov	cx,ydiff		; Get line length in cx
	inc	cx
yline1:
	mov	es:[bx],dl		; Draw next point in line
	add	bx,y_unit		; Point offset to next pixel
					;	in Y direction
	mov	ax,error_term		; Check to see if move require
					;	in X direciton
	add	ax,xdiff
	mov	error_term,ax
	sub	ax,ydiff
	jc	yline2			; If not, continue
	mov	error_term,ax
	add	bx,x_unit		; Else, move left or right
					;	one pixel
yline2:
	loop	yline1			; Loop until count (in CX) complete
linedone:
	mov	sp,bp			; Finished!
	pop	bp
	ret
_linedraw	ENDP

;------------------------------------------------------------------------
;
;Prototype:	void boxdraw(int x1, int y1, int x2, int y2, int color,
;			BYTE *screen)
;
;Description:	Draw a box
;
;Input:		x1		- starting X coordinate
;		y1		- starting Y coordinate
;		x2		- ending X coordinate
;		y2		- ending Y coordinate
;		color		- color for box
;
;Modified:	screen		- pointer to screen
;
;Output:	none
;
;Returns:	none
;
;Side effects:	none
;
;------------------------------------------------------------------------
_boxdraw	PROC
	ARG	x1:WORD,y1:WORD,x2:WORD,y2:WORD,color:WORD, \
		scr_off:WORD,scr_seg:WORD
	push	bp
	mov	bp,sp
	push	di
	push	es
	push	cx
	mov	ax,y2			; Test to see if Y2 < Y1
	sub	ax,y1
	jae	drawbox
	mov	ax,y2			; Y2 is less, so a swap is reqrd
	mov	bx,y1
	mov	y2,bx
	mov	y1,ax
drawbox:
	mov	ax,x2			; Test to see if X2 < X1
	sub	ax,x1
	jae	drawboxline
	mov	ax,x2			; X2 is less, so a swap is reqrd
	mov	bx,x1
	mov	x2,bx
	mov	x1,ax
drawboxline:
	mov	ax,scr_seg		; Get screen segment in ax
	mov	es,ax
	mov	ax,y2			; Find offset of bot left corner by..
	mov	bx,ax
	shl	ax,8			; Find y1 * 256
	shl	bx,6			; Find y1 * 64
	add	ax,bx			; Add them together to get y1 * 320
	add	ax,x1			; And add x1 to get pixel offset
	add	ax,scr_off		; Add screen offset
	mov	di,ax			; Move offset to DI
	mov	ax,color		; Setup Color
	mov	ah,al
	cld
	mov	cx,y2			; Find how many lines to draw
	sub	cx,y1
	inc	cx
drawboxloop:
	push	cx			; Save line counter
	push	di			; Save offset onto screen
	mov	cx,x2
	sub	cx,x1			; Get change in x into DX
	inc	cx			; Need to draw to last X coord
	sar	cx,1			; Divide by 2
	jnc	drawhboxline2		; If even number of bytes, jump
	stosb				; Wasn't, so draw first one then..
drawhboxline2:
	rep	stosw			; Draw the line
	pop	di			; Restore offset of beginning of line
	sub	di,SCREEN_WIDTH		; Up one line
	pop	cx			; Restore line counter
	loop	drawboxloop
	pop	cx
	pop	es
	pop	di
	mov	sp,bp			; Finished!
	pop	bp
	ret
_boxdraw	ENDP

;------------------------------------------------------------------------
;
;Prototype:	void boxdraw(int x1, int y1, int x2, int y2,
;			BYTE *screen)
;
;Description:	Negates the colors in a box
;			(ie.	0	->	255
;				1	->	254
;				2	->	253)
;
;Input:		x1		- starting X coordinate
;		y1		- starting Y coordinate
;		x2		- ending X coordinate
;		y2		- ending Y coordinate
;
;Modified:	screen		- pointer to screen
;
;Output:	none
;
;Returns:	none
;
;Side effects:	none
;
;------------------------------------------------------------------------
_negatebox      PROC
	ARG	x1:WORD,y1:WORD,x2:WORD,y2:WORD,scr_off:WORD,scr_seg:WORD
	push	bp
	mov	bp,sp
	push	di
	push	es
	push	cx
	mov	ax,y2			; Test to see if Y2 < Y1
	sub	ax,y1
	jae	negbox
	mov	ax,y2			; Y2 is less, so a swap is reqrd
	mov	bx,y1
	mov	y2,bx
	mov	y1,ax
negbox:
	mov	ax,x2			; Test to see if X2 < X1
	sub	ax,x1
	jae	negboxline
	mov	ax,x2			; X2 is less, so a swap is reqrd
	mov	bx,x1
	mov	x2,bx
	mov	x1,ax
negboxline:
	mov	ax,scr_seg		; Get screen segment in ax
	mov	es,ax
	mov	ax,y2			; Find offset of bot left corner by..
	mov	bx,ax
	shl	ax,8			; Find y1 * 256
	shl	bx,6			; Find y1 * 64
	add	ax,bx			; Add them together to get y1 * 320
	add	ax,x1			; And add x1 to get pixel offset
	add	ax,scr_off		; Add screen offset
	mov	di,ax			; Move offset to DI
	cld
	mov	cx,y2			; Find how many lines to draw
	sub	cx,y1
	inc	cx
negboxloop:
	push	cx			; Save line counter
	push	di			; Save offset onto screen
	mov	cx,x2
	sub	cx,x1			; Get change in x into DX
	inc	cx			; Needes to draw to last X coord
	sar	cx,1			; Divide by 2
	jnc	neghboxline2		; If even number of bytes, jump
	mov	al,[es:di]		; Get first byte of line
	neg	al			; Negate first byte of line
	stosb
neghboxline2:
	push	cx
negboxloop3:
	mov	ax,[es:di]		; Get next 2 bytes
	neg	ax			; Negate them
	stosw				; Put them back
	loop	negboxloop3
	pop	cx
	pop	di			; Restore offset of beginning of line
	sub	di,SCREEN_WIDTH		; Up one line
	pop	cx			; Restore line counter
	loop	negboxloop
	pop	cx
	pop	es
	pop	di
	mov	sp,bp			; Finished!
	pop	bp
	ret
_negatebox	ENDP

	END
