/*************************************************************************

	FILE:		MOUSE.CPP

	AUTHOR:		Brian Tegart, C.E.T.

	DATE:		April 30, 1995

	DESCRIPTION:	Code for Mouse class

	COPYRIGHT (c) 1995 Tegart Software

==========================================================================

	Revision:	0.0

	Date:		April 30, 1995

	By:		Brian Tegart, C.E.T.

	Action:		Creation

**************************************************************************/

/************/
/* Includes */
/************/
#include "mouse.h"
#include "mouseasm.h"

/***********/
/* Defines */
/***********/

/********************/
/* Global variables */
/********************/

/**********************/
/* External variables */
/**********************/

/******************************/
/* Static function prototypes */
/******************************/

/********************/
/* Static functions */
/********************/

/********************/
/* Public functions */
/********************/

/*------------------------------------------------------------------------

Prototype:	Mouse::Mouse();

Description:	Mouse constructor

Input:		none

Modified:	none

Output:		none

Returns:	none

Side effects:	none

-------------------------------------------------------------------------*/
Mouse::Mouse()
{
	if(initmouse() == 0)
		Initialized = FALSE;
	else
		Initialized = TRUE;
	Visible = FALSE;
} // Mouse Constructor

/*------------------------------------------------------------------------

Prototype:	Mouse::Show();

Description:	Show mouse

Input:		none

Modified:	none

Output:		none

Returns:	none

Side effects:	none

-------------------------------------------------------------------------*/
void Mouse::Show()
{
	if((!Initialized) || Visible)
		return;
	disppointer();
	Visible = TRUE;
} // Mouse::Show

/*------------------------------------------------------------------------

Prototype:	Mouse::Hide();

Description:	Hide mouse

Input:		none

Modified:	none

Output:		none

Returns:	none

Side effects:	none

-------------------------------------------------------------------------*/
void Mouse::Hide()
{
	if((!Initialized) || (!Visible))
		return;
	rempointer();
	Visible = FALSE;
} // Mouse::Hide

/*------------------------------------------------------------------------

Prototype:	Mouse::GetRelPos(int *dx, int *dy);

Description:	Get mouse position relative to last call

Input:		none

Modified:	none

Output:		dx	- pointer to hold change in x
		dy	- pointer to hold change in y

Returns:	none

Side effects:	none

-------------------------------------------------------------------------*/
void Mouse::GetRelPos(int *dx, int *dy)
{
	if(!Initialized)
	{
		*dx = 0;
		*dy = 0;
		return;
	}
	relpos(dx, dy);
} // Mouse::GetRelPos

/*------------------------------------------------------------------------

Prototype:	Mouse::GetPos(int *x, int *y);

Description:	Get mouse position

Input:		none

Modified:	none

Output:		x	- pointer to hold x
		y	- pointer to hold y

Returns:	none

Side effects:	none

-------------------------------------------------------------------------*/
void Mouse::GetPos(int *x, int *y)
{
	if(!Initialized)
	{
		*x = 0;
		*y = 0;
		return;
	}
	getmousepos(x, y);
} // Mouse::GetPos

/*------------------------------------------------------------------------

Prototype:	Mouse::SetPos(int x, int y);

Description:	Set mouse position

Input:		none

Modified:	none

Output:		x	- new x coordinate
		y	- new y coordinate

Returns:	none

Side effects:	none

-------------------------------------------------------------------------*/
void Mouse::SetPos(int x, int y)
{
	if(!Initialized)
		return;
	setmousepos(x, y);
} // Mouse::SetPos

/*------------------------------------------------------------------------

Prototype:	Mouse::ReadButtons();

Description:	Read mouse buttons

Input:		none

Modified:	none

Output:		none

Returns:	MLEFTBUTTON_DOWN, MRIGHTBUTTON_DOWN, MMIDDLEBUTTON_DOWN
		(these may be 'or'ed together), or MNOBUTTON_DOWN

Side effects:	none

-------------------------------------------------------------------------*/
int Mouse::ReadButtons()
{
	if(!Initialized)
		return MNOBUTTON_DOWN;
	return readmbutton();
} // Mouse::ReadButtons

/*------------------------------------------------------------------------

Prototype:	Mouse::MaybeShow();

Description:	Maybe re-show the mouse (see notes)

Input:		none

Modified:	none

Output:		none

Returns:	none

Side effects:	none

Notes:		To be used after MaybeHide - if mouse was visible before
		call to MaybeHide, reshow it. Good to keep the mouse
		hidden if it was previously hidden.

-------------------------------------------------------------------------*/
void Mouse::MaybeShow()
{
	if(!Initialized)
		return;
	if(Visible)
		disppointer();
} // Mouse::MaybeShow

/*------------------------------------------------------------------------

Prototype:	Mouse::MaybeHide();

Description:	Hide the mouse without changing the visible flag (see notes)

Input:		none

Modified:	none

Output:		none

Returns:	none

Side effects:	none

Notes:		To be used before MaybeShow to temporarily hide the mouse
		and reshow it if it was previously visible - good to keep
		to mouse hidden if it was previously hidden

-------------------------------------------------------------------------*/
void Mouse::MaybeHide()
{
	if((!Initialized) || (!Visible))
		return;
	if(Visible)
		rempointer();
} // Mouse::MaybeHide()
