;*************************************************************************
;
;	FILE:		MOUSEASM.ASM
;
;	AUTHOR:		Brian Tegart, C.E.T.
;
;	DATE:		April 30, 1995
;
;	DESCRIPTION:	Mouse routines
;
;	COPYRIGHT (c) 1995 Tegart Software
;
;	NOTES:		Some source used from the book "Flights of Fanatasy"
;
;=========================================================================
;
;	Revision:	0.0
;
;	Date:		April 30, 1995
;
;	By:		Brian Tegart, C.E.T.
;
;	Action:		Creation and modification from old file
;
;*************************************************************************

;**************
;* Directives *
;**************
.MODEL	large
.CODE

;************
;* Includes *
;************

;*************************
;* Function declarations *
;*************************
PUBLIC	_initmouse,_readmbutton,_rempointer,_relpos,_disppointer
PUBLIC	_getmousepos,_setmousepos

;***********
;* Equates *
;***********

;**************
;* Structures *
;**************

;***************
;* Global data *
;***************

;*****************
;* External data *
;*****************

;********************
;* Public functions *
;********************

;------------------------------------------------------------------------
;
;Prototype:	int initmouse();
;
;Description:   Initialize mouse
;
;Input:		none
;
;Modified:	none
;
;Output:	none
;
;Returns:	none
;
;Side effects:	none
;
;------------------------------------------------------------------------
_initmouse	PROC
	xor	ax,ax			; Function zero (initialize)
	int	33h			; Call mouse driver
	ret
_initmouse	ENDP

;------------------------------------------------------------------------
;
;Prototype:	int disppointer();
;
;Description:   Display mouse pointer
;
;Input:		none
;
;Modified:	none
;
;Output:	none
;
;Returns:	none
;
;Side effects:	none
;
;------------------------------------------------------------------------
_disppointer	PROC
	mov	ax,1			; Function 1 (display pointer)
	int	33h			; Call mouse driver
	ret
_disppointer	ENDP

;------------------------------------------------------------------------
;
;Prototype:	int rempointer();
;
;Description:   Remove mouse pointer
;
;Input:		none
;
;Modified:	none
;
;Output:	none
;
;Returns:	none
;
;Side effects:	none
;
;------------------------------------------------------------------------
_rempointer	PROC
	mov	ax,2			; Function 2 (remove pointer)
	int	33h			; Call mouse driver
	ret
_rempointer	ENDP

;------------------------------------------------------------------------
;
;Prototype:	int readmbutton();
;
;Description:   Read mouse button positions
;
;Input:		none
;
;Modified:	none
;
;Output:	none
;
;Returns:	Mouse button positions
;
;Side effects:	none
;
;------------------------------------------------------------------------
_readmbutton	PROC
	mov	ax,3			; Function 3 (read buttons)
	int	33h			; Call mouse driver
	mov	ax,bx			; Return result
	ret
_readmbutton	ENDP

;------------------------------------------------------------------------
;
;Prototype:	void relpos(int *x, int *y);
;
;Description:   Get change in mouse position relative to last call
;
;Input:		none
;
;Modified:	none
;
;Output:	x	- pointer to hold change in X
;		y	- pointer to hold change in Y
;
;Returns:	none
;
;Side effects:	none
;
;------------------------------------------------------------------------
_relpos   PROC
	ARG	x:DWORD,y:DWORD
	push	bp
	mov	bp,sp
	mov	ax,000bh		; Request function 0bh
					;	(relative mouse position)
	int	33h			; Call mouse driver
	shr	cx,1			; X pos comes back in CX doubled, fix
	les	bx,x			; Point es:bx at x parameter
	mov	[es:bx],cx		; ...and store relative position
	les	bx,y			; Point es:bx at y parameter
	mov	[es:bx],dx		; ...and store relative position
	pop	bp
	ret
_relpos   ENDP

;------------------------------------------------------------------------
;
;Prototype:	void getmousepos(int *x, int *y);
;
;Description:   Get mouse position
;
;Input:		none
;
;Modified:	none
;
;Output:	x	- pointer to hold X coordinate
;		y	- pointer to hold Y coordinate
;
;Returns:	none
;
;Side effects:	none
;
;------------------------------------------------------------------------
_getmousepos   PROC
	ARG	x:DWORD,y:DWORD
	push	bp
	mov	bp,sp
	mov	ax,0003h		; Request function 0bh
					;	(relative mouse position)
	int	33h			; Call mouse driver
	shr	cx,1			; X pos comes back doubled, so /2
	les	bx,x			; Point es:bx at x parameter
	mov	[es:bx],cx		; ...and store relative position
	les	bx,y			; Point es:bx at y parameter
	mov	[es:bx],dx		; ...and store relative position
	pop	bp
	ret
_getmousepos   ENDP

;------------------------------------------------------------------------
;
;Prototype:	void setmousepos(int x, int y);
;
;Description:   Set mouse position
;
;Input:		none
;
;Modified:	none
;
;Output:	x	- X coordinate
;		y	- Y coordinate
;
;Returns:	none
;
;Side effects:	none
;
;------------------------------------------------------------------------
_setmousepos   PROC
	ARG	x:WORD,y:WORD
	push	bp
	mov	bp,sp
	mov	ax,0004h		; Request function 0bh
					;	(relative mouse position)
	mov	cx,x
	shl	cx,1			; Double X coordinate
	mov	dx,y
	int	33h			; Call mouse driver
	pop	bp
	ret
_setmousepos   ENDP

	END
