#include <dos.h>
#include <string.h>
#include "mycolors.h"
#include "screen.h"
#include "wait.h"

void FadeToBlack(BYTE *screen, unsigned long delay_time)
{
	BYTE origpalette[256*3];
	BYTE fadepalette[256*3];
	int done, i;

	// get the current palette
	getpalette(origpalette);

	memcpy(fadepalette, origpalette, 256*3);

	do {
		done = 1;
		for(i=0;i<256*3;i++)
		{
			if(fadepalette[i])
			{
				fadepalette[i]--;
				done = 0;
			}
		}
		// wait for scan to prevent snow
		WaitForRetrace();
		setpalette(fadepalette, 0, 256);
		if(delay_time)
			Wait(delay_time);
	} while(!done);

	cls(screen);
	setpalette(origpalette, 0, 256);
} // FadeToBlack

void RotatePalette(int startcol, int endcol, int rotate)
{
	BYTE origpalette[256*3];
	BYTE newpalette[256*3];
	BYTE temprgb[3];

	getpalette(origpalette);
	memcpy(newpalette, origpalette, 256*3);

	int reg = startcol;
	int nextreg = reg + rotate;
	memcpy(temprgb, &(newpalette[startcol*3]), 3);
	while(reg != endcol)
	{
		if(nextreg>=256)
			nextreg-=256;
		if(nextreg<0)
			nextreg+=256;
		memcpy(&(newpalette[reg*3]), &(newpalette[nextreg*3]), 3);
		reg += rotate;
		if(reg>=256)
			reg-=256;
		if(reg<0)
			reg+=256;
		nextreg = reg + rotate;
	};
	memcpy(&(newpalette[endcol*3]), temprgb, 3);
	WaitForRetrace();
	setpalette(newpalette, 0, 256);
} // RotattePalette

// Palette Constructor 1
Palette::Palette(BYTE (*InitRGB)[3])
{
	RGB = new BYTE[256][3];

	for(int i=0;i<256;i++)
	{
		RGB[i][0] = InitRGB[i][0];
		RGB[i][1] = InitRGB[i][1];
		RGB[i][2] = InitRGB[i][2];
	}
} // Palette constructor 1

// Palette Constructor 2
Palette::Palette(BYTE *InitRGB)
{
	RGB = new BYTE[256][3];

	memcpy(&(RGB[0][0]), InitRGB, 256*3);
} // Palette constructor 2

// Palette::Set makes this sets the VGA palette to this objects palette
void Palette::Set()
{
	setpalette(&(RGB[0][0]), 0, 256);
} // Palette::Set

// Palette::Reset puts new numbers into this objects palette array
void Palette::Reset(BYTE *NewRGB)
{
	memcpy(&(RGB[0][0]), NewRGB, 256*3);
} // Palette::Reset

// Palette Destructor
Palette::~Palette()
{
	delete [3] RGB;
} // Palette Destructor
