#ifndef _PCX_H
#define _PCX_H

#include <stdio.h>

#include "verify.h"

#ifndef MAXX
#define MAXX 319
#define MAXY 199
#endif

#ifndef BYTE
typedef unsigned char	BYTE;
typedef unsigned int	WORD;
#endif

#define TOO_MANY_ARGUMENTS	3

#define PCX_MANUFACTURER	0x0A
#define PCX_VERSION         5
#define PCX_ENCODING		1
#define PCX_256PALETTE		0x0C

struct pcx_header {
	BYTE manufacturer;		// Always 0x0a for PCX files
	BYTE version;			// Always 5 for 256-colors files
	BYTE encoding;			// Always 1
	BYTE bits_per_pixel;	// Always 8 for 256-color files
	int xmin, ymin;			// Coordinates for topleft corner
	int xmax, ymax;			// Width and height of image
	int hres;				// Horizontal res of image
	int vres;				// Vertical res of image
	BYTE palette16[48];		// EGA palette; not used for 256-color files
	BYTE reserved;			// Reserved for future use
	BYTE color_planes;		// Color planes
	int bytes_per_line;		// Number of bytes in 1 line of pixels
	int palette_type;		// Should be 2 for color palette
	BYTE filler[58];		// Makes header 128 bytes long
}; // struct pcx_header

struct pcx_struct {
	pcx_header header;
	BYTE *image;	// Pointer to a buffer holding the 64Kbyte bitmap
	BYTE *cimage;	// Pointer to a buffer holding a compressed
					//	version of the bitmap
	BYTE palette[3*256];	// Array holding the 768-byte palette
	long clength;			// Length of the compressed bitmap
}; // struct pcx_struct

// Class Pcx is for using 256-color VGA PCX files
class Pcx
{
	private:
		BYTE *temp;						// used for saving screen before show
		int Visible;
		struct pcx_struct *Pcx_Struct;
		int image_loaded;
		int infile;						// file handle for file
		FILE *fp;
		int load_image(int pcxfile);	// Bitmap loading func
		void save_image();
		void compress();				// Compress pcx func
		BYTE *screen;
	public:
		Pcx(BYTE *InitScreen);
		int load(char *filename);		// Load pcx data
		int save(char *filename);
		BYTE *GetImage()
		{ return Pcx_Struct->image; };
		BYTE *GetPalette()
		{ return Pcx_Struct->palette; };
		void load_palette(int pcxfile);	// Palette loading func
		void save_palette(FILE *fp);
		int New();            // create a new, empty PCX instead of loading
		int Show();
		void Hide();
		~Pcx();
};

#endif
