#include <dos.h>
#include <string.h>
#include "evntmngr.h"
#include "font.h"
#include "keyboard.h"
#include "mouse.h"
#include "popup.h"
#include "scancode.h"
#include "screen.h"

extern int event_mask;
extern Mouse mouse;

EntryBox::EntryBox(int InitX, int InitY,
		int InitColor, int InitTopColor, int InitBottomColor,
		BYTE *InitScreen, FontStruct *InitFontPtr,
		int InitEntryX, int InitEntryY,
		int InitTextColor, int InitEntryColor,
		char *InitDescription, int InitMaxLength) :
	TextButton(InitEntryX, InitEntryY, InitEntryX, InitEntryY,
			InitColor, InitTopColor, InitBottomColor, NOSAVE_BG,
			InitScreen, InitDescription, InitFontPtr, InitTextColor,
			CENTER_TEXT, CENTER_TEXT, 1)
{
	MaxLength = InitMaxLength;
	EntryColor = InitEntryColor;
	DescX = InitX;
	DescY = InitY;
	EntryX = InitEntryX;
	EntryY = InitEntryY;
	CursPos = 0;
	Entry = new char[InitMaxLength+1];
	Entry[0] = 0;
	MoveTo(InitEntryX-3, InitEntryY-3,
		InitEntryX+3+(MaxLength*InitFontPtr->width),
		InitEntryY+InitFontPtr->height + 2);
} // EntryBox constructor

void EntryBox::Show()
{
	if(!Visible)
	{
		mouse.MaybeHide();
		GraphString(FontPtr, DescX, DescY+3, TextColor, GetText(), screen);
		mouse.MaybeShow();
		Button::Show();
	}
	mouse.MaybeHide();
	GraphString(FontPtr, EntryX, EntryY, EntryColor, Entry, screen);
	mouse.MaybeShow();
} // EntryBox::Show

void EntryBox::ShowCursor()
{
	int cx1 = X1 + 3 + (CursPos * FontPtr->width);
	mouse.MaybeHide();
	hlinedraw(cx1, Y2-2, cx1+FontPtr->width,
		EntryColor, screen);
	mouse.MaybeShow();
} // EntryBox::ShowCursor

void EntryBox::HideCursor()
{
	int cx1 = X1 + 3 + (CursPos * FontPtr->width);
	mouse.MaybeHide();
	hlinedraw(cx1, Y2-2, cx1+FontPtr->width, Color, screen);
	mouse.MaybeShow();
} // EntryBox::HideCursor

void EntryBox::HighLight()
{
	Button::Push();
	ShowCursor();
} // EntryBox::HighLight

void EntryBox::UnHighLight()
{
	Button::Release();
	HideCursor();
} // EntryBox::UnHighLight

void EntryBox::HideChar(int Offset)
{
	char nuked[2];

	nuked[0] = Entry[Offset];
	nuked[1] = 0;
	mouse.MaybeHide();
	GraphString(FontPtr, EntryX+(Offset*FontPtr->width), EntryY,
		Color, nuked, screen);
	mouse.MaybeShow();
} // EntryBox::HideChar

void EntryBox::FixDisplay(WORD SCode)
{
	int Len = strlen(Entry);
	int i;

	HideCursor();
	switch(SCode)
	{
		case SCODEBACKSPACE:
			if(!CursPos)
			{
				ShowCursor();
				return;
			}
			CursPos--;
			for(i=CursPos;i<Len;i++)
				HideChar(i);
			for(i=CursPos;i<Len+1;i++)
				Entry[i] = Entry[i+1];
			break;
		case SCODEDELETE:
			if(CursPos == Len)
				break;
			for(i=CursPos;i<Len;i++)
				HideChar(i);
			for(i=CursPos;i<Len;i++)
				Entry[i] = Entry[i+1];
			if(CursPos >= strlen(Entry))
				CursPos--;
			if(CursPos<=0)
				CursPos=0;
			break;
	}
	Show();
	ShowCursor();
} // EntryBox::FixDisplay

void EntryBox::HandleKey(WORD SCode)
{
	BYTE Ascii = (BYTE)SCode;   // Get the low byte: the ascii value

	HideCursor();
	if((Ascii >= 0x20) && (Ascii <= 0x7E))  // if a regular ascii character
	{
		if(CursPos!=MaxLength)  // if not at the max length
		{
			if(Entry[CursPos])      // if already a character there..
				HideChar(CursPos);
			else
				Entry[CursPos+1] = 0;
			Entry[CursPos] = Ascii; // add the character
			CursPos++;
		}
		if(CursPos >= MaxLength)
			CursPos = MaxLength-1;
		Show();
		ShowCursor();
		return;
	}
	else
	{
		switch(SCode)
		{
			case SCODELEFT:
				if(CursPos)
					CursPos--;
				break;
			case SCODERIGHT:
				if(CursPos<strlen(Entry))
					CursPos++;
				break;
			case SCODEHOME:
				CursPos = 0;
				break;
			case SCODEEND:
				CursPos = strlen(Entry);
				break;
		}
	}
	if(CursPos >= MaxLength)
		CursPos = MaxLength-1;
	FixDisplay(SCode);
} // EntryBox::HandleKey

void EntryBox::SetEntry(char *NewEntry)
{
	strcpy(Entry, NewEntry);
	CursPos = strlen(Entry);
	if(CursPos >= MaxLength)
		CursPos = MaxLength-1;
	if(Visible)
	{
		ColorHide(Color);
		Show();
	}
}

EntryBox::~EntryBox()
{
	delete Entry;
	if(Visible)
		Hide();
} // EntryBox::~EntryBox

PopUp::PopUp(int InitX1, int InitY1, int InitX2, int InitY2,
		int InitColor, int InitTopColor, int InitBottomColor,
		BYTE *InitScreen, FontStruct *InitFontPtr,
		char *InitText, int InitTextColor, int InitButtons,
		int InitYOffset, int InitLines) :
	TextButton(InitX1, InitY1, InitX2, InitY2,
		InitColor, InitTopColor, InitBottomColor, SAVE_BG,
		InitScreen, InitText, InitFontPtr, InitTextColor, CENTER_TEXT,
		TOP_TEXT, InitLines)
{
	int bX1[MAX_BUTTONS];
	int bY1[MAX_BUTTONS];
	int bX2[MAX_BUTTONS];
	int bY2[MAX_BUTTONS];
	char Words[MAX_BUTTONS][10];
	int SectionWidth;
	int ButtonWidth;
	int i;

	NumPushButtons = 0;
	Buttons = InitButtons;
	if(InitButtons & BUT_YES)
	{
		ButtonVals[NumPushButtons] = BUT_YES;
		strcpy(Words[NumPushButtons++], " Yes  ");
	}
	if(InitButtons & BUT_NO)
	{
		ButtonVals[NumPushButtons] = BUT_NO;
		strcpy(Words[NumPushButtons++], "  No  ");
	}
	if(InitButtons & BUT_OK)
	{
		ButtonVals[NumPushButtons] = BUT_OK;
		strcpy(Words[NumPushButtons++], "  OK  ");
	}
	if(InitButtons & BUT_CANCEL)
	{
		ButtonVals[NumPushButtons] = BUT_CANCEL;
		strcpy(Words[NumPushButtons++], "Cancel");
	}

	SectionWidth = (InitX2 - InitX1)/NumPushButtons;
	ButtonWidth = (strlen("CANCEL") * FontPtr->width) + Lines + Lines + 6;

	for(i=0;i<NumPushButtons;i++)
	{
		bY2[i] = InitY2 - InitYOffset - InitLines;
		bY1[i] = bY2[i] - InitLines - InitYOffset - InitYOffset
			- FontPtr->height;
		if(!i)
			bX1[i] = InitX1 + SectionWidth/2 - ButtonWidth/2;
		else
			bX1[i] = bX1[i-1] + SectionWidth;
		bX2[i] = bX1[i] + ButtonWidth;
		PushButtons[i] = new TextButton(bX1[i], bY1[i], bX2[i], bY2[i],
			InitColor, InitTopColor, InitBottomColor, NOSAVE_BG,
			InitScreen, Words[i], InitFontPtr, InitTextColor, CENTER_TEXT,
			CENTER_TEXT, InitLines);
	}
} // PopUp constructor

void PopUp::Show()
{
	if(Visible)
		return;
	TextButton::Show();
	for(int i=0;i<NumPushButtons;i++)
		PushButtons[i]->Show();
} // PopUp::Show

int PopUp::GetResult()
{
	int event_type, event, result;

	Show();
	do {
		getevent(event_mask, &event_type, &event);
		result = HandleEvent(event_type, event);
	} while(!result);
	Hide();
	return result;
} // PopUp::GetResult

void PopUp::Hide()
{
	if(!Visible)
		return;
	for(int i=0;i<NumPushButtons;i++)
		PushButtons[i]->ColorHide(Color);
	TextButton::Hide();
} // PopUp::Hide

int PopUp::HandleEvent(int Event_Type, int Event)
{
	int bx1,bx2,by1,by2,mx,my;
	int i,j;

	if((Event_Type & MOUSE_EVENT)&&
		((Event & LBUTTONDOWN)||(Event & LBUTTONHELD)||(Event & LBUTTONUP)))
	{
		for(i=0;i<NumPushButtons;i++)
		{
			PushButtons[i]->GetCoords(&bx1, &by1, &bx2, &by2);
			mouse.GetPos(&mx, &my);
			if((mx>=bx1)&&(mx<=bx2)&&(my>=by1)&&(my<=by2)) // if this button
			{
				// release all others
				for(j=0;j<NumPushButtons;j++)
					if(j!=i)
						PushButtons[j]->Release();
				if(Event & LBUTTONUP)
				{
					PushButtons[i]->Release();
					return ButtonVals[i];
				}
				else
					PushButtons[i]->Push();
			}
			else
				PushButtons[i]->Release();
		}
	}
	else if(Event_Type & KEYBOARD_EVENT)
	{
		switch(Event)
		{
			case SCODEENTER:
				return ButtonVals[0];
				break;
			case SCODEAY:
				if(Buttons & BUT_YES)
					return BUT_YES;
				break;
			case SCODEAN:
				if(Buttons & BUT_NO)
					return BUT_NO;
				break;
			case SCODEAO:
				if(Buttons & BUT_OK)
					return BUT_OK;
				break;
			case SCODEAC:
				if(Buttons & BUT_CANCEL)
					return BUT_CANCEL;
				break;
			case SCODEESC:
				return ButtonVals[NumPushButtons-1];
				break;
		}
	} // end of if keyboard event
	return 0;                    // Nothing to do
} // PopUp::HandleEvent

PopUp::~PopUp()
{
	for(int i=0;i<NumPushButtons;i++)
	{
		PushButtons[i]->Hide();
		PushButtons[i]->TextButton::~TextButton();
	}
	if(Visible)
		Hide();
} // PopUp destructor

InfoPanel::InfoPanel(int InitX1, int InitY1, int InitX2, int InitY2,
		int InitColor, int InitTopColor, int InitBottomColor,
		BYTE *InitScreen, FontStruct *InitFontPtr,
		char *InitText, int InitTextColor, int InitButtons,
		int InitYOffset, EntryBox **InitEntries, int InitNumEntries,
		int InitLines) :
	PopUp(InitX1, InitY1, InitX2, InitY2, InitColor, InitTopColor,
		InitBottomColor, InitScreen, InitFontPtr, InitText,
		InitTextColor, InitButtons, InitYOffset, InitLines)
{
	Entries = InitEntries;
	NumEntries = InitNumEntries;
} // InfoPanel constructor

void InfoPanel::Show()
{
	int i;

	if(Visible)
		return;
	PopUp::Show();
	for(i=0;i<NumEntries;i++)
		Entries[i]->Show();
	ActiveEntry = 0;
	Entries[ActiveEntry]->HighLight();
} // InfoPanel::Show

void InfoPanel::Hide()
{
	if(!Visible)
		return;
	for(int i=0;i<NumEntries;i++)
		Entries[i]->ColorHide(Color);
	PopUp::Hide();
} // InfoPanel::Hide

int InfoPanel::HandleEvent(int Event_Type, int Event)
{
	int bx1,bx2,by1,by2,mx,my;
	int result;
	int i;

	result = PopUp::HandleEvent(Event_Type, Event);
	if(result)
		return result;

	if((Event_Type & MOUSE_EVENT)&&
		((Event & LBUTTONDOWN)||(Event & LBUTTONHELD)||(Event & LBUTTONUP)))
	{
		for(i=0;i<NumEntries;i++)
		{
			Entries[i]->GetCoords(&bx1, &by1, &bx2, &by2);
			mouse.GetPos(&mx, &my);
			// if in this entry
			if((mx>=bx1)&&(mx<=bx2)&&(my>=by1)&&(my<=by2))
			{
				if(i!=ActiveEntry)
				{
					Entries[ActiveEntry]->UnHighLight();
					ActiveEntry = i;
					Entries[i]->HighLight();
				}
			}
		} // end of for loop
	}
	else if(Event_Type & KEYBOARD_EVENT)
	{
		switch(Event)
		{
			case SCODEENTER:
				return 1;
			case SCODEESC:
				return 2;
			case SCODETAB:
				Entries[ActiveEntry]->UnHighLight();
				ActiveEntry++;
				if(ActiveEntry==NumEntries)
					ActiveEntry=0;
				Entries[ActiveEntry]->HighLight();
				break;
			case SCODESTAB:
				Entries[ActiveEntry]->UnHighLight();
				ActiveEntry--;
				if(ActiveEntry<0)
					ActiveEntry=NumEntries-1;
				Entries[ActiveEntry]->HighLight();
				break;
			default:
				Entries[ActiveEntry]->HandleKey(Event); // temp);
				return 0;
		}
	} // end of if keyboard event
	return 0;                    // Nothing to do
} // InfoPanel::HandleEvent

void InfoPanel::GetEntries(char **InfoEntries)
{
	int i;

	for(i=0;i<NumEntries;i++)
		InfoEntries[i] = Entries[i]->GetEntry();
} // InfoPanel::GetEntries

void InfoPanel::SetEntry(int EntryNum, char *EntryText)
{
	Entries[EntryNum]->SetEntry(EntryText);
} // InfoPanel::SetEntry

InfoPanel::~InfoPanel()
{
	if(Visible)
		Hide();
} // InfoPanel::~InfoPanel
