;*************************************************************************
;
;	FILE:		SCREEN.ASM
;
;	AUTHOR:		Brian Tegart, C.E.T.
;
;	DATE:		April 29, 1995
;
;	DESCRIPTION:	Mode 0x13 screen functions
;
;	COPYRIGHT (c) 1995 Tegart Software
;
;=========================================================================
;
;	Revision:	0.0
;
;	Date:		April 29, 1995
;
;	By:		Brian Tegart, C.E.T.
;
;	Action:		Creation and modification from old file
;
;*************************************************************************

;**************
;* Directives *
;**************
.286
.MODEL large
.CODE

;************
;* Includes *
;************

;*************************
;* Function declarations *
;*************************
PUBLIC _cls, _fillscreen, _setgmode, _setpalette, _getpalette, _setcolor
PUBLIC _getbox, _putbox, _putboxtransparent, _transferbox, _copyscreen
PUBLIC _WaitForRetrace

;***********
;* Equates *
;***********
SCREEN_WIDTH	equ	320
SCREEN_HEIGHT	equ	200
SCREEN_AREA	equ	SCREEN_WIDTH*SCREEN_HEIGHT

;**************
;* Structures *
;**************

;***************
;* Global data *
;***************

;*****************
;* External data *
;*****************

;********************
;* Public functions *
;********************

;------------------------------------------------------------------------
;
;Prototype:	void cls(BYTE *screen_adr);
;
;Description:	Clear screen buffer (64000 bytes)
;
;Input:		screen_adr	- pointer to screen buffer
;
;Modified:	none
;
;Output:	none
;
;Returns:	none
;
;Side effects:	none
;
;------------------------------------------------------------------------
_cls	PROC
	ARG	screen:DWORD
	push	bp			;Save BP
	mov	bp,sp			;Set up stack pointer
	push	di			;Save DI
	les	di,screen		;Point ES:DI at screen
	mov	cx,SCREEN_AREA/2	;Number of words
	xor	ax,ax			;Store 0s
	rep	stosw			; in video memory
	pop	di
	pop	bp
	ret
_cls	ENDP

;------------------------------------------------------------------------
;
;Prototype:	void fillscreen(BYTE *screen_adr, int color);
;
;Description:	Fill screen buffer (64000 bytes) with color
;
;Input:		screen_adr	- pointer to screen buffer
;		color		- color (0-255) to fill with
;
;Modified:	none
;
;Output:	none
;
;Returns:	none
;
;Side effects:	none
;
;------------------------------------------------------------------------
_fillscreen	PROC
	ARG	screen:DWORD,fillcolor:WORD
	push	bp			;Save BP
	mov	bp,sp			;Set up stack pointer
	push	di			;Save DI
	les	di,screen		;Point ES:DI at screen
	mov	cx,SCREEN_AREA/2	;Number of words
	mov	ax,fillcolor		;Store color
	rep	stosw			; in video memory
	pop	di
	pop	bp
	ret
_fillscreen	ENDP

;------------------------------------------------------------------------
;
;Prototype:	void setgmode(int mode);
;
;Description:	Set VGA adapter to BIOS graphics mode
;
;Input:		mode		- graphics mode to set to
;
;Modified:	none
;
;Output:	none
;
;Returns:	none
;
;Side effects:	none
;
;------------------------------------------------------------------------
_setgmode	PROC
	ARG	mode:WORD
	push	bp			;Save BP
	mov	bp,sp			;Set up stack pointer
	mov	ax,mode			;AL = video mode
	xor	ah,ah			;AH = function number
	int	10h			;Call int 10
	pop	bp			;Restore BP
	ret
_setgmode	ENDP

;------------------------------------------------------------------------
;
;Prototype:	void setpalette(BYTE *color_regs, int firstreg,
;			int numregs);
;
;Description:	Set VGA palette registers
;
;Input:		color_regs	- pointer to array of register values
;		firstreg	- palette reg to start at
;		numregs		- number of palette regs to set
;
;Modified:	none
;
;Output:	none
;
;Returns:	none
;
;Side effects:	none
;
;------------------------------------------------------------------------
_setpalette	PROC
	ARG	regs:DWORD, firstreg:WORD, numregs:WORD
	push	bp			;Save BP
	mov	bp,sp			;Set up stack pointer
	les	dx,regs			;Point ES:DX at palette regs
	mov	ax,1012h  		;BIOS function 10h, sub func 12h
	mov	bx,firstreg		;Start with first reg
	mov	cx,numregs		;Set all 256 (100h) regs
	int	10h			;Call video BIOS
	pop	bp
	ret
_setpalette	ENDP

;------------------------------------------------------------------------
;
;Prototype:	void getpalette(BYTE *buffer);
;
;Description:	Get VGA palette registers
;
;Input:		none
;
;Modified:	none
;
;Output:	buffer		- pointer to hold palette register
;					values - must be 768 bytes
;
;Returns:	none
;
;Side effects:	none
;
;------------------------------------------------------------------------
_getpalette	PROC
	ARG	buffer:DWORD
	push	bp			;Save BP
	mov	bp,sp			;Set up stack pointer
	push	ax
	push	bx
	push	cx
	push	dx
	push	es
	mov	ax,1017h		;BIOS function 10h, sub func 17h
	xor	bx,bx			;Starting color is 0
	mov	cx,256			;Number of color regs
	les	dx,buffer		;Point ES:DX at palette regs
	int	10h			;Call video BIOS
	pop	es
	pop	dx
	pop	cx
	pop	bx
	pop	ax
	pop	bp
	ret
_getpalette	ENDP

;------------------------------------------------------------------------
;
;Prototype:	void setcolor(int colornum, BYTE red, BYTE green,
;			BYTE blue);
;
;Description:	Set one VGA palette register
;
;Input:		colornum	- palette reg (0-255) for the new color
;		red		- red value (0-63)
;		green		- green value (0-63)
;		blue		- blue value (0-63)
;
;Modified:	none
;
;Output:	none
;
;Returns:	none
;
;Side effects:	none
;
;------------------------------------------------------------------------
_setcolor	PROC
	ARG	colornum:WORD, red:BYTE, green:BYTE, blue:BYTE
	push	bp			;Save BP
	mov	bp,sp			;Set up stack pointer
	mov     ah,10h
	mov     al,10h
	mov     bx,colornum
	mov     ch,green
	mov     cl,blue
	mov     dh,red
	int	10h			;Call video BIOS
	pop	bp
	ret
_setcolor	ENDP

;------------------------------------------------------------------------
;
;Prototype:	void getbox(BYTE *dest, int x1, int y1, int numcols,
;			int numrows, BYTE *screen);
;
;Description:	Copies a rectangular region from a screen buffer
;			to a linear buffer
;
;Input:		x1		- x coordinate (top-left) of box to copy
;		y1		- y coordinate (top-left) of box to copy
;		numcols		- number of columns to copy (width)
;		numrows		- number of rows to copy (height)
;		screen		- pointer to screen buffer containing
;					source box
;
;Modified:	none
;
;Output:	dest		- pointer to buffer to hold linear bitmap
;					Must be allocated by caller
;
;Returns:	none
;
;Side effects:	none
;
;------------------------------------------------------------------------
_getbox PROC
	ARG	dest:DWORD,x1:WORD,y1:WORD,numcols:WORD,numrows:WORD, \
		screen:DWORD
	push	bp
	mov	bp,sp
	push	ds
	push	es
	push	di
	push	si
	lds	si,screen		; Point DS:SI at screen (source)
	add	si,x1
	mov	ax,y1
	mov	bx,y1
	shl	ax,8			; Find y * 256
	shl	bx,6			; Find y * 64
	add	ax,bx			; Add them together to get y1 * 320
	add	si,ax			; Add to SI to point DS:SI at coord
	les	di,dest			; Point ES:DI at Dest buffer
	cld
	mov	cx,numrows
savenextline:
	push	cx
	mov	cx,numcols
	sar	cx,1			; Divide by 2
	jnc	getrestofline		; If even number of bytes, jump
	movsb				; Wasn't, so draw first one then..
getrestofline:
	rep	movsw			; Move next two pixels the buffer
	sub	si,numcols
	add	si,SCREEN_WIDTH		; Point SI at next line
	pop	cx
	loop	savenextline
	pop	si
	pop	di
	pop	es
	pop	ds
	mov	sp,bp
	pop	bp
	ret
_getbox ENDP

;------------------------------------------------------------------------
;
;Prototype:	void putbox(BYTE *buff, int x1, int y1, int numcols,
;			int numrows, BYTE *screen);
;
;Description:	Copies a previously saved (by getbox) rectangular region
;			in the form of a linear buffer back to a screen
;			buffer
;
;Input:         buff		- pointer to saved linear buffer
;		x1		- x coordinate (top-left) of box to copy
;		y1		- y coordinate (top-left) of box to copy
;		numcols		- number of columns to copy (width)
;		numrows		- number of rows to copy (height)
;
;Modified:	screen		- pointer to screen buffer
;
;Output:	none
;
;Returns:	none
;
;Side effects:	none
;
;------------------------------------------------------------------------
_putbox PROC
	ARG	buff:DWORD,x1:WORD,y1:WORD,numcols:WORD,numrows:WORD, \
		screen:DWORD
	push	bp
	mov	bp,sp
	push	ds
	push	es
	push	di
	push	si
	les	di,screen		; Point ES:DI at screen (dest)
	add	di,x1
	mov	ax,y1
	mov	bx,y1
	shl	ax,8			; Find y * 256
	shl	bx,6			; Find y * 64
	add	ax,bx			; Add them together to get y1 * 320
	add	di,ax			; Add to SI to point DS:SI at coord
	lds	si,buff			; Point DS:SI at buffer
	cld
	mov	cx,numrows
putnextline:
	push	cx
	mov	cx,numcols
	sar	cx,1			; Divide by 2
	jnc	putrestofline		; If even number of bytes, jump
	movsb				; Wasn't, so draw first one then..
putrestofline:
	rep	movsw			; move next two pixels to buffer
	sub	di,numcols
	add	di,SCREEN_WIDTH		; Point DI at next line
	pop	cx
	loop	putnextline
	pop	si
	pop	di
	pop	es
	pop	ds
	mov	sp,bp
	pop	bp
	ret
_putbox ENDP

;------------------------------------------------------------------------
;
;Prototype:	void putboxtransparent(BYTE *buff, int x1, int y1,
;			int numcols, int numrows, BYTE *screen);
;
;Description:	Copies a previously saved (by getbox) rectangular region
;			in the form of a linear buffer back to a screen
;			buffer, but not placing color 0 pixels
;
;Input:         buff		- pointer to saved linear buffer
;		x1		- x coordinate (top-left) of box to copy
;		y1		- y coordinate (top-left) of box to copy
;		numcols		- number of columns to copy (width)
;		numrows		- number of rows to copy (height)
;
;Modified:	screen		- pointer to screen buffer
;
;Output:	none
;
;Returns:	none
;
;Side effects:	none
;
;------------------------------------------------------------------------
_putboxtransparent PROC
	ARG	buff:DWORD,x1:WORD,y1:WORD,numcols:WORD,numrows:WORD, \
		screen:DWORD
	push	bp
	mov	bp,sp
	push	ds
	push	es
	push	di
	push	si
	les	di,screen		; Point ES:DI at screen (dest)
	add	di,x1
	mov	ax,y1
	mov	bx,y1
	shl	ax,8			; Find y * 256
	shl	bx,6			; Find y * 64
	add	ax,bx			; Add them together to get y1 * 320
	add	di,ax			; Add to SI to point DS:SI at coord
	lds	si,buff			; Point DS:SI at buffer
	cld
	mov	cx,numrows
putnextxparentline:
	push	cx
	mov	cx,numcols
putrestofxparentline:
	lodsb
	cmp	al,0			; Is the byte a '0'?
	jz	isxparent		; if yes, jump
	stosb				; save the byte
	jmp	donexparentpixel
isxparent:				; Isn't a 0, so it wasn't stored,
	inc	di			;  so we have to inc DI manually
donexparentpixel:
	loop	putrestofxparentline   	; Finish off the line
	sub	di,numcols
	add	di,SCREEN_WIDTH		; Point DI at next line
	pop	cx
	loop	putnextxparentline
	pop	si
	pop	di
	pop	es
	pop	ds
	mov	sp,bp
	pop	bp
	ret
_putboxtransparent ENDP

;------------------------------------------------------------------------
;
;Prototype:	void transferbox(int destx1, int desty1, BYTE *destscreen,
;			int srcx1, int srcy1, BYTE *srcscreen,
;            		int numcols, int numrows);
;
;Description:	Copies a box from one screen buffer to another
;
;Input:         destx1		- x coordinate to place box on destscreen
;		desty1		- y coordinate to place box on destscreen
;		srcx1		- x coordinate to copy box from srcscreen
;		srcy1		- y coordinate to copy box from srcscreen
;		srcscreen	- pointer to source screen buffer
;		numcols		- number of columns to copy (width)
;		numrows		- number of rows to copy (height)
;
;Modified:	destscreen	- pointer to destination screen buffer
;
;Output:	none
;
;Returns:	none
;
;Side effects:	none
;
;------------------------------------------------------------------------
_transferbox PROC
	ARG	destx1:WORD,desty1:WORD,destscreen:DWORD,srcx1:WORD, \
		srcy1:WORD,srcscreen:DWORD,numcols:WORD,numrows:WORD
	push	bp
	mov	bp,sp
	push	ds
	push	es
	push	di
	push	si
	les	di,destscreen		; Point ES:DI at destscreen (dest)
	add	di,destx1
	mov	ax,desty1
	mov	bx,desty1
	shl	ax,8			; Find y * 256
	shl	bx,6			; Find y * 64
	add	ax,bx			; Add them together to get y1 * 320
	add	di,ax			; Add to DI to point ES:DI at coord
	lds	si,srcscreen		; Point DS:SI at srcscreen (source)
	add	si,srcx1
	mov	ax,srcy1
	mov	bx,srcy1
	shl	ax,8			; Find y * 256
	shl	bx,6			; Find y * 64
	add	ax,bx			; Add them together to get y1 * 320
	add	si,ax			; Add to SI to point DS:SI at coord
	cld
	mov	cx,numrows		; Loop for number of rows
transfernextline:
	push	cx			; save for use after mov
	mov	cx,numcols		; how many pixels per row
	sar	cx,1			; Divide by 2
	jnc	transferrestofline	; If even number of bytes, jump
	movsb				; Wasn't, so draw first one then..
transferrestofline:
	rep	movsw			; draw next two pixels
	sub	di,numcols
	add	di,SCREEN_WIDTH		; Point DI at next line
	sub	si,numcols
	add	si,SCREEN_WIDTH		; Point SI at next line
	pop	cx
	loop	transfernextline
	pop	si
	pop	di
	pop	es
	pop	ds
	mov	sp,bp
	pop	bp
	ret
_transferbox ENDP

;------------------------------------------------------------------------
;
;Prototype:	void copyscreen(BYTE *destscreen, BYTE *srcscreen);
;
;Description:	Copies one screen buffer to another
;
;Input:		srcscreen	- pointer to source screen buffer
;
;Modified:	destscreen	- pointer to destination screen buffer
;
;Output:	none
;
;Returns:	none
;
;Side effects:	none
;
;------------------------------------------------------------------------
_copyscreen PROC
	ARG	destscreen:DWORD,srcscreen:DWORD
	push	bp
	mov	bp,sp
	push	ds
	push	es
	push	di
	push	si
	les	di,destscreen		; Point ES:DI at destscreen (dest)
	lds	si,srcscreen		; Point DS:SI at srcscreen (source)
	cld
	mov	cx,SCREEN_AREA/2	; Loop for number of words
	rep	movsw			; copy next two pixels
	pop	si
	pop	di
	pop	es
	pop	ds
	mov	sp,bp
	pop	bp
	ret
_copyscreen ENDP

;------------------------------------------------------------------------
;
;Prototype:	void WaitForRetrace();
;
;Description:	Wait for vertical retrace
;
;Input:		none
;
;Modified:	none
;
;Output:	none
;
;Returns:	none
;
;Side effects:	none
;
;------------------------------------------------------------------------
_WaitForRetrace PROC
	mov	dx,3dah
VRT:
	in	al,dx
	test	al,8
	jnz	VRT			; this waits until retrace starts
NoVRT:
	in	al,dx
	test	al,8
	jz	NoVRT			; this waits until it ends
	ret
_WaitForRetrace ENDP

	END
