#include <dos.h>
#include <string.h>
#include "font.h"
#include "popup.h"
#include "screen.h"
#include "setup.h"

extern EntryBox *SaveAsBoxes[];
extern EntryBox *SavePaletteBoxes[];
extern EntryBox *OpenPaletteBoxes[];
extern EntryBox *QuickAnimateEntries[];
extern EntryBox *FullAnimateEntries[];
extern EntryBox *CopyBoxes[];

extern InfoPanel *SaveAsPanel;
extern InfoPanel *SavePalettePanel;
extern InfoPanel *OpenPalettePanel;
extern InfoPanel *QuickAnimPanel;
extern InfoPanel *FullAnimPanel;
extern InfoPanel *CopyPanel;

void SetUpInitHeaders(char ***InitHeaders)
{
	int NumMenuStrings = 35;    // change this value as the menu changes
	int i;
	*InitHeaders = new char *[NumMenuStrings];
	for(i=0;i<NumMenuStrings;i++)
		(*InitHeaders)[i] = new char[20];

	strcpy((*InitHeaders)[0], "File");
	strcpy((*InitHeaders)[1], "Bit-Map");
	strcpy((*InitHeaders)[2], "Colors");
	strcpy((*InitHeaders)[3], "Animation");
	strcpy((*InitHeaders)[4], "Misc");
	strcpy((*InitHeaders)[5], "");

	strcpy((*InitHeaders)[6], "Open");
	strcpy((*InitHeaders)[7], "New");
	strcpy((*InitHeaders)[8], "Save");
	strcpy((*InitHeaders)[9], "Save As ");
	strcpy((*InitHeaders)[10], "Open Palette");
	strcpy((*InitHeaders)[11], "Save Palette");
	strcpy((*InitHeaders)[12], "Save Palette As ");
	strcpy((*InitHeaders)[13], "Exit");
	strcpy((*InitHeaders)[14], "");

	strcpy((*InitHeaders)[15], "Copy");
	strcpy((*InitHeaders)[16], "Clear");
	strcpy((*InitHeaders)[17], "Color Fill");
	strcpy((*InitHeaders)[18], "Scroll Left");
	strcpy((*InitHeaders)[19], "Scroll Right");
	strcpy((*InitHeaders)[20], "Scroll Up");
	strcpy((*InitHeaders)[21], "Scroll Down");
	strcpy((*InitHeaders)[22], "Flip Horizontal ");
	strcpy((*InitHeaders)[23], "Flip Vertical");
	strcpy((*InitHeaders)[24], "Invert");
	strcpy((*InitHeaders)[25], "");

	strcpy((*InitHeaders)[26], "Edit Color");
	strcpy((*InitHeaders)[27], "");

	strcpy((*InitHeaders)[28], "Quick Animate ");
	strcpy((*InitHeaders)[29], "Full Animate");
	strcpy((*InitHeaders)[30], "");

	strcpy((*InitHeaders)[31], "Grid On");
	strcpy((*InitHeaders)[32], "Grid Off ");
	strcpy((*InitHeaders)[33], "View");
	strcpy((*InitHeaders)[34], "");
} // SetUpInitHeaders

void SetUpFont(FontStruct *Font)
{
	WORD fontseg, fontoff;
	char bytesperchar;

	GetFontAddr(ROM8x8, &fontseg, &fontoff, &bytesperchar);
	Font->width = 8;
	Font->height = 8;
	Font->bytesperchar = bytesperchar;
	Font->fptr = (BYTE *)MK_FP(fontseg, fontoff);
} // SetUpFont

void SetUpScreen(BYTE **screen, int *oldmode)
{
	*screen = (BYTE *)MK_FP(0xa000, 0);
	cls(*screen);							// Clear the screen
	*oldmode = *(char *)MK_FP(0x40,0x49);	// Save old mode
	setgmode(0x13);
} // SetUpScreen

void SetUpEntryBoxes(BYTE *screen, FontStruct *Font, int textcolor,
	int entrycolor)
{
	SaveAsBoxes[0] = new EntryBox(35, 45, L_GRAY, WHITE, GRAY,
		screen, Font, 40, 60, textcolor, entrycolor, "Filename:", 30);
	OpenPaletteBoxes[0] = new EntryBox(35, 45, L_GRAY, WHITE, GRAY,
		screen, Font, 40, 60, textcolor, entrycolor, "Filename:", 30);
	SavePaletteBoxes[0] = new EntryBox(35, 45, L_GRAY, WHITE, GRAY,
		screen, Font, 40, 60, textcolor, entrycolor, "Filename:", 30);
	QuickAnimateEntries[0] = new EntryBox(35, 45, L_GRAY, WHITE, GRAY,
		screen, Font, 40, 60, textcolor, entrycolor, "Sequence:", 30);
	FullAnimateEntries[0] = new EntryBox(35, 25, L_GRAY, WHITE, GRAY,
		screen, Font, 40, 40, textcolor, entrycolor, "Left Sequence:", 30);
	FullAnimateEntries[1] = new EntryBox(35, 55, L_GRAY, WHITE, GRAY,
		screen, Font, 40, 70, textcolor, entrycolor, "Right Sequence:", 30);
	FullAnimateEntries[2] = new EntryBox(35, 85, L_GRAY, WHITE, GRAY,
		screen, Font, 40, 100, textcolor, entrycolor, "Up Sequence:", 30);
	FullAnimateEntries[3] = new EntryBox(35, 115, L_GRAY, WHITE, GRAY,
		screen, Font, 40, 130, textcolor, entrycolor, "Down Sequence:", 30);
	FullAnimateEntries[4] = new EntryBox(35, 145, L_GRAY, WHITE, GRAY,
		screen, Font, 140, 145,	textcolor, entrycolor, "Stop Sprite:", 2);
	CopyBoxes[0] = new EntryBox(115, 70, L_GRAY, WHITE, GRAY,
		screen, Font, 165, 70, textcolor, entrycolor, "From:", 2);
	CopyBoxes[1] = new EntryBox(115, 85, L_GRAY, WHITE, GRAY,
		screen, Font, 165, 85, textcolor, entrycolor, "To:", 2);
} // SetUpEntryBoxes

void SetUpInfoPanels(BYTE *screen, FontStruct *Font, int textcolor)
{
	SaveAsPanel = new InfoPanel(25, 25, 295, 110,
		L_GRAY, WHITE, GRAY, screen, Font, "Save As",
		textcolor, BUT_OKCANCEL, 3, SaveAsBoxes,
		SAVEAS_ENTRIES, DEFAULT_LINES);
	OpenPalettePanel = new InfoPanel(25, 25, 295, 110,
		L_GRAY, WHITE, GRAY, screen, Font, "Open Palette",
		textcolor, BUT_OKCANCEL, 3, OpenPaletteBoxes, OPENPALETTE_ENTRIES,
		DEFAULT_LINES);
	SavePalettePanel = new InfoPanel(25, 25, 295, 110,
		L_GRAY, WHITE, GRAY, screen, Font, "Save Palette",
		textcolor, BUT_OKCANCEL, 3, SavePaletteBoxes, SAVEPALETTE_ENTRIES,
		DEFAULT_LINES);
	QuickAnimPanel = new InfoPanel(25, 25, 295, 110,
		L_GRAY, WHITE, GRAY, screen, Font, "Quick Animate",
		textcolor, BUT_OKCANCEL, 3, QuickAnimateEntries, QUICKANIM_ENTRIES,
		DEFAULT_LINES);
	FullAnimPanel = new InfoPanel(25, 5, 295, 185,
		L_GRAY, WHITE, GRAY, screen, Font, "Full Animate",
		textcolor, BUT_OKCANCEL, 3, FullAnimateEntries, FULLANIM_ENTRIES,
		DEFAULT_LINES);
	CopyPanel = new InfoPanel(75, 45, 245, 130,
		L_GRAY, WHITE, GRAY, screen, Font, "Copy Sprite",
		textcolor, BUT_OKCANCEL, 3, CopyBoxes, COPY_ENTRIES,
		DEFAULT_LINES);
} // SetUpInfoPanels
