/*************************************************************************

	FILE:       MYSHAPES.CPP

	AUTHOR:     Brian Tegart, C.E.T.

	DATE:       April 30, 1995

	DESCRIPTION:    Code for basic shapes

	COPYRIGHT (c) 1995 Tegart Software

==========================================================================

	Revision:   0.0

	Date:       April 30, 1995

	By:     Brian Tegart, C.E.T.

	Action:     Creation and modification from old file

**************************************************************************/

/************/
/* Includes */
/************/
#include <alloc.h>
#include <conio.h>
#include <dos.h>
#include <mem.h>
#include <stdio.h>
#include <string.h>
#include "font.h"
#include "keyboard.h"
#include "mouse.h"
#include "shapes.h"
#include "pcx.h"
#include "screen.h"
#include "sprite.h"

/***********/
/* Defines */
/***********/

/********************/
/* Global variables */
/********************/

/**********************/
/* External variables */
/**********************/
extern Mouse mouse;

/******************************/
/* Static function prototypes */
/******************************/

/********************/
/* Static functions */
/********************/

/*------------------------------------------------------------------------

Prototype:

Description:

Input:

Modified:

Output:

Returns:

Side effects:

-------------------------------------------------------------------------*/

/********************/
/* Public functions */
/********************/

// Rectangle Constructor
Rectangle::Rectangle(int InitX1, int InitY1, int InitX2, int InitY2,
			int InitColor, int InitSaveBGState, BYTE *InitScreen)
{
	X1 = InitX1;
	Y1 = InitY1;
	X2 = InitX2;
	Y2 = InitY2;
	Color = InitColor;
	screen = InitScreen;
	SaveBG = InitSaveBGState;
	SavedBG = 0;
	Visible = 0;
	background = NULL;
} // Rectangle Constructor

void Rectangle::SaveBackground()
{
	if(SavedBG)
		delete background;
	background = new BYTE[(X2-X1+1) * (Y2-Y1+1)];
	mouse.MaybeHide();
	getbox(background,X1,Y1,X2-X1+1,Y2-Y1+1,screen);
	mouse.MaybeShow();
	SavedBG = 1;
} // Rectangle::SaveBackground

void Rectangle::RestoreBackground()
{
	if(!SavedBG)
		return;
	mouse.MaybeHide();
	putbox(background,X1,Y1,X2-X1+1,Y2-Y1+1,screen);
	mouse.MaybeShow();
	delete background;
	SavedBG = 0;
} // Rectangle::RestoreBackground

void Rectangle::MoveTo(int NewX1, int NewY1, int NewX2, int NewY2)
{
	int WasVis = 0;

	if(Visible)
	{
		Hide();
		WasVis = 1;
	}
	X1 = NewX1;
	Y1 = NewY1;
	X2 = NewX2;
	Y2 = NewY2;
	if(WasVis)
		Show();
} // Rectangle::MoveTo

void Rectangle::Show()
{
	if(Visible)
		return;
	if(SavedBG)
		delete background;
	if(SaveBG)
		SaveBackground();
	mouse.MaybeHide();
	boxdraw(X1, Y1, X2, Y2, Color, screen);
	mouse.MaybeShow();
	Visible = 1;
} // Rectangle::Show

void Rectangle::Hide()
{
	if(!Visible)
		return;
	mouse.MaybeHide();
	if(SavedBG)
		RestoreBackground();
	else
		boxdraw(X1, Y1, X2, Y2, 0, screen);
	mouse.MaybeShow();
	Visible = 0;
} // Rectangle::Hide

void Rectangle::ColorHide(int HideColor)
{
	if(!Visible)
		return;
	mouse.MaybeHide();
	if(SavedBG)
		RestoreBackground();
	else
		boxdraw(X1, Y1, X2, Y2, HideColor, screen);
	mouse.MaybeShow();
	Visible = 0;
} // Rectangle::ColorHide

Rectangle::~Rectangle()
{
	Hide();
} // Rectangle::~Rectangle

// Grid Constructor
Grid::Grid(int InitAcross, int InitDown, int InitX1, int InitY1,
		int InitX2, int InitY2, int InitColor, BYTE *InitScreen)
	: Rectangle(InitX1, InitY1, InitX2, InitY2, InitColor, NOSAVE_BG,
		InitScreen)
{
	int XOff, YOff;
	int ActX1, ActY1, ActX2, ActY2;

	if(InitAcross<=0)
		Across = 1;
	else
		Across = InitAcross;
	if(InitDown<=0)
		Down = 1;
	else
		Down = InitDown;

	CellWidth = (InitX2 - InitX1) / Across;
	CellHeight = (InitY2 - InitY1) / Down;

	XOff = ((InitX2-InitX1) - (CellWidth*Across)) / 2;
	YOff = ((InitY2-InitY1) - (CellHeight*Down)) / 2;

	ActX1 = InitX1 + XOff;
	ActY1 = InitY1 + YOff;
	ActX2 = ActX1 + (Across * CellWidth);
	ActY2 = ActY1 + (Down * CellHeight);
	MoveTo(ActX1, ActY1, ActX2, ActY2);
} // Grid Constructor

void Grid::Show()
{
	int i;
	int Color = GetColor();

	// Draw Vertical Lines
	for(i=0;i<Across+1;i++)
		vlinedraw(X1+(i*CellWidth), Y1, Y2, Color, screen);

	// Draw Horizontal Lines
	for(i=0;i<Down+1;i++)
		hlinedraw(X1, Y1+(i*CellHeight), X2, Color, screen);
} // Grid::Show

// Button Constructor
Button::Button(int InitX1, int InitY1, int InitX2, int InitY2,
		int InitColor, int InitTopColor, int InitBottomColor,
		int InitSaveBGState, BYTE *InitScreen, int InitLines)
	: Rectangle(InitX1, InitY1, InitX2, InitY2, InitColor, InitSaveBGState,
		InitScreen)
{
	TopColor = InitTopColor;
	BotColor = InitBottomColor;
	Lines = InitLines;
	Pressed = 1;
} // Button Constructor

void Button::Show()
{
	Pressed = 1;
	Rectangle::Show();
	Release();
} // Button::Show

void Button::Push()
{
	int X1, Y1, X2, Y2;
	int Lin = Lines;

	if(Pressed==1)
		return;
	GetCoords(&X1, &Y1, &X2, &Y2);
	mouse.MaybeHide();
	do {
		vlinedraw(X1, Y2-1, Y1, BotColor, screen);
		hlinedraw(X1, Y1, X2, BotColor, screen);
		vlinedraw(X2, Y1+1, Y2, TopColor, screen);
		hlinedraw(X2--, Y2--, X1++, TopColor, screen);
		Y1++;
	} while(--Lin);
	mouse.MaybeShow();
	Pressed = 1;
} // Button::Push

void Button::Release()
{
	int X1, Y1, X2, Y2;
	int Lin = Lines;

	if(!Pressed)
		return;
	GetCoords(&X1, &Y1, &X2, &Y2);
	mouse.MaybeHide();
	do {
		vlinedraw(X1, Y2-1, Y1, TopColor, screen);
		hlinedraw(X1, Y1, X2, TopColor, screen);
		vlinedraw(X2, Y1+1, Y2, BotColor, screen);
		hlinedraw(X2--, Y2--, X1++, BotColor, screen);
		Y1++;
	} while(--Lin);
	mouse.MaybeShow();
	Pressed = 0;
} // Button::Release

Button::~Button()
{
	Hide();
} // Button::~Button

// TextButton Constructor

TextButton::TextButton(int InitX1, int InitY1, int InitX2, int InitY2,
			int InitColor, int InitTopColor, int InitBottomColor,
			int InitSaveBGState, BYTE *InitScreen,
			char *InitText, FontStruct *InitFontPtr,
			int InitTextColor, int InitXJust, int InitYJust, int InitLines)
	: Button(InitX1, InitY1, InitX2, InitY2, InitColor, InitTopColor,
		InitBottomColor, InitSaveBGState, InitScreen, InitLines)
{
	TextColor = InitTextColor;
	FontPtr = InitFontPtr;
	X_Just = InitXJust;
	Y_Just = InitYJust;
	Text = new char[strlen(InitText)+1];
	strcpy(Text, InitText);
} // TextButton Constructor

void TextButton::Show()
{
	int Len = strlen(Text);
	int XPos, YPos;

	if(Visible)
		return;

	Button::Show();
	if(Text[0])
	{
		switch(X_Just)
		{
			case LEFT_TEXT:
				XPos = X1 + Lines + 3;
				break;
			case RIGHT_TEXT:
				XPos = X2 - Lines - 3 - (strlen(Text)*FontPtr->width);
				break;
			default:  // case CENTER_TEXT:
				XPos = ((X1+X2)/2)-(Len*(FontPtr->width / 2));
				break;
		}
		switch(Y_Just)
		{
			case TOP_TEXT:
				YPos = Y1 + Lines + 3;
				break;
			case BOTTOM_TEXT:
				YPos = Y2 - Lines - 3 - FontPtr->height;
				break;
			default:  // case CENTER_TEXT:
				YPos = ((Y1+Y2)/2) - (FontPtr->height / 2) + 1;
				break;
		}
		mouse.MaybeHide();
		GraphString(FontPtr, XPos, YPos, TextColor, Text, screen);
		mouse.MaybeShow();
	}
} // TextButton::Show

void TextButton::Hide()
{
	if(!Visible)
		return;
	Button::Hide();
} // TextButton::Hide

TextButton::~TextButton()
{
	delete Text;
	Hide();
} // TextButton::~TextButton
