/*************************************************************************

	FILE:		SHAPES.H

	AUTHOR:		Brian Tegart, C.E.T.

	DATE:		April 30, 1995

	DESCRIPTION:	Include file for basic shapes

	COPYRIGHT (c) 1995 Tegart Software

==========================================================================

	Revision:	0.0

	Date:		April 30, 1995

	By:		Brian Tegart, C.E.T.

	Action:		Creation and modification from old file

**************************************************************************/

#ifndef _MYSHAPES_H
#define _MYSHAPES_H

/************/
/* Includes */
/************/
#include "font.h"
#include "line.h"

/***********/
/* Defines */
/***********/
#ifndef MAXX
#define MAXX   	319
#define MAXY	199
#endif

#define DEFAULT_LINES	2

#define NOSAVE_BG	0
#define SAVE_BG		1

#define CENTER_TEXT	0
#define LEFT_TEXT	1
#define RIGHT_TEXT	2
#define TOP_TEXT        1
#define BOTTOM_TEXT	2

/***************************/
/* Typedefs and structures */
/***************************/
#ifndef BYTE
typedef unsigned char	BYTE;
typedef unsigned int	WORD;
#endif

/*********************/
/* Public prototypes */
/*********************/

// Declaration of Rectangle class
class Rectangle
{
	protected:
		int X1, Y1, X2, Y2;
		int Color;
		int SaveBG;	// specifies whether background is to be saved
		int SavedBG;	//    "         "	  "     is currently saved
		int Visible;
		BYTE *screen;
		BYTE *background;
		void SaveBackground();
		void RestoreBackground();
	public:
		Rectangle(int InitX1, int InitY1, int InitX2, int InitY2,
			int InitColor, int InitSaveBGState, BYTE *InitScreen);
		virtual void Show();
		virtual void Hide();
		virtual void ColorHide(int HideColor);
		void GetCoords(int *gX1, int *gY1, int *gX2, int *gY2)
		{ *gX1 = X1; *gY1 = Y1; *gX2 = X2; *gY2 = Y2; };
		int GetColor()
		{ return Color; };
		void SetColor(int NewColor)
		{ Color = NewColor; };
		void MoveTo(int NewX1, int NewY1, int NewX2, int NewY2);
		~Rectangle();
}; // Rectangle class declaration

// Declaration of Grid class
class Grid : public Rectangle
{
	protected:
		int Across, Down;
		int CellWidth, CellHeight;
	public:
		Grid(int InitAcross, int InitDown, int InitX1, int InitY1,
			int InitX2, int InitY2, int InitColor, BYTE *InitScreen);
		void Show();
}; // Grid class declaration

// Declaration of Button class
class Button : public Rectangle
{
	protected:
		int TopColor;
		int BotColor;
		int Lines;
		int Pressed;
	public:
		Button(int InitX1, int InitY1, int InitX2, int InitY2, int InitColor,
			int InitTopColor, int InitBottomColor, int InitSaveBGState,
			BYTE *InitScreen, int InitLines = DEFAULT_LINES);
		void GetColors(int *TC, int *BC)
		{ *TC = TopColor; *BC = BotColor; };
		int GetNumLines()
		{ return Lines; };
		void Show();
		void Push();
		void Release();
		~Button();
}; // Button class declaration

// Declaration of TextButton Class
class TextButton : public Button
{
	protected:
		char *Text;
		FontStruct *FontPtr;
		int TextColor;
		int X_Just;
		int Y_Just;
	public:
		TextButton(int InitX1, int InitY1, int InitX2, int InitY2,
			int InitColor, int InitTopColor, int InitBottomColor,
			int InitSaveBGState, BYTE *InitScreen,
			char *InitText, FontStruct *InitFontPtr,
			int InitTextColor, int InitXJust = CENTER_TEXT,
			int InitYJust = TOP_TEXT, int InitLines = DEFAULT_LINES);
		void Show();
		void Hide();
		char *GetText()
		{ return Text; };
		~TextButton();
}; // TextButton class declaration

#endif
