#include <alloc.h>
#include <dos.h>
#include <mem.h>
#include <stdio.h>
#include "screen.h"
#include "sprite.h"

// Sprite constructor
// Allocates memory for NUM_SPRITES sprite structures
Sprite::Sprite(int num_sprites, int w, int h, BYTE *InitScreen,
	BYTE *InitShadow)
{
	int j;

	screen = InitScreen;
	shadow = InitShadow;
	NumSprites = num_sprites;
	images = new BYTE *[num_sprites];
	width = w;
	height = h;
	xsprite = 0;
	ysprite = 0;
	xbg = 0;
	ybg = 0;
	savebuffer = new BYTE [width*height];
	for(j=0;j<NumSprites;j++)
		images[j] = new BYTE [width*height];
} // Sprite constructor

// Sprite::puttransparent draws sprite on screen with upper lefthand corner
// at coordinates x,y.  Zero pixels are treated as transparent
void Sprite::puttransparent(int sprite_num, int x, int y, BYTE *screen)
{
	getbox(savebuffer, x, y, width, height, screen);
	putboxtransparent(images[sprite_num], x, y,	width, height, screen);
	xsprite = x;
	ysprite = y;
	xbg = x;
	ybg = y;
} // Sprite::puttransparent

// Sprite::put draws sprite on screen with upper lefthand corner at
//	coordinates x,y.  Zero pixels are NOT treated as transparent
void Sprite::put(int sprite_num, int x, int y, BYTE *screen)
{
	getbox(savebuffer, x, y, width, height, screen);
	putbox(images[sprite_num], x, y, width, height,	screen);
	// Record current coordinates of sprite
	xsprite = x;
	ysprite = y;
} // Sprite::put

// Sprite::Move moves a sprite flicker-free, where dx and dy are deltas
//              from current position
void Sprite::Move(int dx, int dy, int image_num)
{
	putbox(savebuffer, xbg, ybg, width, height, shadow);
	xsprite += dx;
	ysprite += dy;
	if(xsprite<0)
		xsprite = 0;
	else if(xsprite>320-width)
		xsprite = 320-width;
	if(ysprite<0)
		ysprite = 0;
	else if(ysprite>200-height)
		ysprite = 200-height;
	getbox(savebuffer, xsprite, ysprite, width, height,
		shadow);
	putboxtransparent(images[image_num], xsprite, ysprite,
		width, height, shadow);
	transferbox(min(xsprite, xbg), min(ysprite, ybg), screen,
		min(xsprite, xbg), min(ysprite, ybg), shadow,
		width+ABS(dx), height+ABS(dy));
	xbg = xsprite;
	ybg = ysprite;
} // Sprite::Move

void Sprite::Copy(int destnum, int sourcenum)
{
	size_t num = width;
	num *= height;
	if((destnum>=0)&&(destnum<NumSprites)
		&&(sourcenum>=0)&&(sourcenum<NumSprites))
	memmove(images[destnum], images[sourcenum], num);
} // Sprite::Copy

Sprite::~Sprite()
{
	int j;

	delete savebuffer;
	for(j=0;j<NumSprites;j++)
		delete images[j];
	delete images;
} // Sprite destructor

