#include <conio.h>
#include <stdio.h>
#include "animfunc.h"
#include "evntmngr.h"
#include "font.h"
#include "pcx.h"
#include "popup.h"
#include "rtrncode.h"
#include "sprite.h"
#include "tegart.h"
#include "wait.h"

int ChecksumTegart()
{
	FILE *fp;
	unsigned char checksum = 0;
	unsigned char bytein;

	fp = fopen(TEGART_FILENAME, "r");
	if(fp == NULL)
		return NO_FILE_OPEN;

	while(!feof(fp))
	{
		fread(&bytein,1,1,fp);
		checksum ^= bytein;
	}

	fclose(fp);

	if(checksum == TEGART_CHECKSUM)
		return GOOD;
	return BAD;
}

int TitleScreen(BYTE *screen, BYTE *shadow, FontStruct *Font,
	int errortextcolor)
{
	Pcx *TegartPcx = new Pcx(screen);
	Sprite *TegartSprite = new Sprite(NUM_TEGART_SPRITES,
		TEGART_WIDTH, TEGART_HEIGHT, screen, shadow);
	int rc;
	int sx,sy;
	int i;
	unsigned long int wait_time = WAIT_SMALLEST;

	rc = ChecksumTegart();
	if(rc!=GOOD)
	{
		switch(rc)
		{
			case NO_FILE_OPEN:
				PopMessage("Cannot open TEGART.DAT", BUT_OK, screen, Font,
					errortextcolor);
				delete TegartPcx;
				delete TegartSprite;
				return rc;
			case BAD:
				PopMessage("TEGART.DAT is corrupt", BUT_OK, screen, Font,
					errortextcolor);
				delete TegartPcx;
				delete TegartSprite;
				return rc;
		}
	}

	rc = TegartPcx->load(TEGART_FILENAME);
	if(rc==NO_MEM)
	{
		PopMessage("Not enough memory", BUT_OK, screen, Font,
				errortextcolor);
		delete TegartPcx;
		delete TegartSprite;
		return rc;
	}

	for(i=0,sx=0,sy=0;i<NUM_TEGART_SPRITES;i++,sx+=TEGART_WIDTH)
		TegartSprite->grab(TegartPcx->GetImage(), i, sx, sy);

	Sprite *SoftwareSprite = new Sprite(NUM_SOFTWARE_SPRITES,
		 SOFTWARE_WIDTH, SOFTWARE_HEIGHT, screen, shadow);
	for(i=0,sx=0;i<NUM_SOFTWARE_SPRITES;i++,sx+=SOFTWARE_WIDTH)
		SoftwareSprite->grab(TegartPcx->GetImage(), i, sx, 160);

	setpalette(TegartPcx->GetPalette(), 0, 256);
	delete TegartPcx;

	cls(screen);
	copyscreen(shadow, screen);

	int k=0;
	for(i=0;i<200;i++)
	{
		hlinedraw(0,i,319,255-k,shadow);
		if((i) && (!(i%4)))
			k++;
	}

	for(i=0;i<NUM_TEGART_SPRITES;i++)
		TegartSprite->puttransparent(i,
			(320-(NUM_TEGART_SPRITES*TEGART_WIDTH))/2+(i*TEGART_WIDTH) + 5,
			55, shadow);

	BYTE rowcolor;
	for(unsigned z=0;z<64000;z++)
	{
		if(!(z%320))
			rowcolor = shadow[z];
		if(shadow[z] < 64)
			shadow[z] = rowcolor - 0x0c;
	}
	for(i=0;i<NUM_TEGART_SPRITES;i++)
		TegartSprite->puttransparent(i,
			(320-(NUM_TEGART_SPRITES*TEGART_WIDTH))/2+(i*TEGART_WIDTH),
			50, shadow);
	copyscreen(screen, shadow);
	// 'S'
	SoftwareSprite->puttransparent(0, 304, 120, shadow);
	for(i=304;i>=25;i-=2)
	{
		SoftwareSprite->Move(-2, 0, 0);
		Wait(wait_time);
	}
	// 'E'
	SoftwareSprite->SetNewCoords(0, 120);
	SoftwareSprite->puttransparent(7, 0, 120, shadow);
	for(i=0;i<13+(7*(SOFTWARE_WIDTH+10));i+=2)
	{
		SoftwareSprite->Move(2, 0, 7);
		Wait(wait_time);
	}
	// 'O'
	SoftwareSprite->SetNewCoords(304, 120);
	SoftwareSprite->puttransparent(1, 304, 120, shadow);
	for(i=304;i>=25+(1*(SOFTWARE_WIDTH+10));i-=2)
	{
		SoftwareSprite->Move(-2, 0, 1);
		Wait(wait_time);
	}
	// 'R'
	SoftwareSprite->SetNewCoords(0, 120);
	SoftwareSprite->puttransparent(6, 0, 120, shadow);
	for(i=0;i<13+(6*(SOFTWARE_WIDTH+10));i+=2)
	{
		SoftwareSprite->Move(2, 0, 6);
		Wait(wait_time);
	}
	// 'F'
	SoftwareSprite->SetNewCoords(304, 120);
	SoftwareSprite->puttransparent(2, 304, 120, shadow);
	for(i=304;i>=25+(2*(SOFTWARE_WIDTH+10));i-=2)
	{
		SoftwareSprite->Move(-2, 0, 2);
		Wait(wait_time);
	}
	// 'A'
	SoftwareSprite->SetNewCoords(0, 120);
	SoftwareSprite->puttransparent(5, 0, 120, shadow);
	for(i=0;i<13+(5*(SOFTWARE_WIDTH+10));i+=2)
	{
		SoftwareSprite->Move(2, 0, 5);
		Wait(wait_time);
	}
	// 'T'
	SoftwareSprite->SetNewCoords(304, 120);
	SoftwareSprite->puttransparent(3, 304, 120, shadow);
	for(i=304;i>=25+(3*(SOFTWARE_WIDTH+10));i-=2)
	{
		SoftwareSprite->Move(-2, 0, 3);
		Wait(wait_time);
	}
	// 'W'
	SoftwareSprite->SetNewCoords(0, 120);
	SoftwareSprite->puttransparent(4, 0, 120, shadow);
	for(i=0;i<13+(4*(SOFTWARE_WIDTH+10));i+=2)
	{
		SoftwareSprite->Move(2, 0, 4);
		Wait(wait_time);
	}

	delete TegartSprite;
	delete SoftwareSprite;

	int event_type, event;
	for(i=0;i<200;i++)	// 4 second delay
	{
		Wait(WAIT_50th);
		getevent(MOUSE_EVENTS + KEYBOARD_EVENTS, &event_type, &event);
		if(event != NOEVENT)
		{
			if(event_type & KEYBOARD_EVENT)
				break;
			if(event_type & MOUSE_EVENT)
				if((event & LBUTTONDOWN) || (event & RBUTTONDOWN))
					break;
		}
	}

	FadeToBlack(screen, WAIT_100th * 3);

	return GOOD;
}
