#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "rtrncode.h"
#include "verify.h"

extern int CritError;

int VerifyOpenEntries(char **Entries, char *filename, int *width,
	int *height, int *numsprites)
{
	FILE *fp;
	unsigned vwidth, vheight, vnum;
	unsigned across, down;
	int toomany = 0;

	fp = fopen(Entries[0], "rb");
	if(fp==NULL)
		return NO_FILE_OPEN;
	fclose(fp);
	vwidth = atoi(Entries[1]);
	if((vwidth<MIN_SPRITE_WIDTH) || (vwidth>MAX_SPRITE_WIDTH))
		return BAD_X_DIMENSION;
	vheight = atoi(Entries[2]);
	if((vheight<MIN_SPRITE_WIDTH) || (vheight>MAX_SPRITE_HEIGHT))
		return BAD_Y_DIMENSION;
	vnum = atoi(Entries[3]);
	if((vnum<MIN_NUM_SPRITES) || (vnum>MAX_NUM_SPRITES))
		return BAD_NUM_SPRITES;
	across = 320 / vwidth;
	down = 200 / vheight;
	if(vnum > across * down)
	{
		vnum = down * across;
		toomany = 1;
	}
	strcpy(filename, Entries[0]);
	*width = (int)vwidth;
	*height = (int)vheight;
	*numsprites = (int)vnum;
	if(toomany)
		return vnum;
	return GOOD;
} // VerifyOpenEntries

int VerifyNewEntries(char **Entries, int *width, int *height,
			  int *numsprites)
{
	unsigned vwidth, vheight, vnum;
	unsigned across,down;
	int toomany = 0;

	vwidth = atoi(Entries[0]);
	if((vwidth<MIN_SPRITE_WIDTH) || (vwidth>MAX_SPRITE_WIDTH))
		return BAD_X_DIMENSION;
	vheight = atoi(Entries[1]);
	if((vheight<MIN_SPRITE_HEIGHT) || (vheight>MAX_SPRITE_HEIGHT))
		return BAD_Y_DIMENSION;
	vnum = atoi(Entries[2]);
	if((vnum<MIN_NUM_SPRITES) || (vnum>MAX_NUM_SPRITES))
		return BAD_NUM_SPRITES;
	across = 320 / vwidth;
	down = 200 / vheight;
	if(vnum > across * down)
	{
		toomany = 1;
		vnum = across * down;
	}
	*width = (int)vwidth;
	*height = (int)vheight;
	*numsprites = (int)vnum;
	if(toomany)
		return vnum;
	return GOOD;
} // VerifyNewEntries

int VerifySaveAsEntries(char **Entries, char *filename)
{
	FILE *fp;

	fp = fopen(Entries[0], "rb");
	strcpy(filename, Entries[0]);
	if(fp==NULL)                 // doesn't exist - OK to create
		return GOOD;
	fclose(fp);                   // already exists
	return FILE_ALREADY_EXISTS;
} // VerifySaveAsEntries

int VerifyOpenPaletteEntries(char **Entries, char *filename)
{
	FILE *fp;

	fp = fopen(Entries[0], "rb");
	if(fp==NULL)
		return NO_FILE_OPEN;
	fclose(fp);                   // exists
	strcpy(filename, Entries[0]);
	return GOOD;
} // VerifySavePaletteEntries

int VerifySavePaletteEntries(char **Entries, char *filename)
{
	FILE *fp;

	fp = fopen(Entries[0], "rb");
	if(fp==NULL)                 // doesn't exist - OK to create
	{
		strcpy(filename, Entries[0]);
		return GOOD;
	}
	fclose(fp);                   // already exists
	return FILE_ALREADY_EXISTS;
} // VerifySavePaletteEntries

int ConvertStringToInts(char *entries, int max_val, int *seqs, int *NumVals)
{
	int stlen;
	int numvals;
	char *temp1;
	char *temp2;
	struct TempStruct{
		int val;
		int range;
	}temp_seqs[50];
	int i,j,k;
	stlen = strlen(entries);

	// check fo valid characters
	for(i=0;i<stlen;i++)
		if( ((entries[i] < '0') || (entries[i] > '9'))
				&& (entries[i] != ' ') && (entries[i] != ',')
				&& (entries[i] != '-'))
			return BAD_SEQUENCE;

	for(i=0;i<50;i++)
	{
		temp_seqs[i].val   = 0;
		temp_seqs[i].range = 0;
	}

	temp1 = entries;
	// strip off leading spaces and whitespace
	while((*temp1 == ' ') || (*temp1 == ','))
		temp1++;
	// strip off trailing spaces
	temp2 = &(temp1[strlen(temp1)-1]);
	while((*temp2 == ' ') || (*temp2 == ','))
		temp2--;
	*(temp2+1) = 0;

	// if first or last character is a '-', error
	if(*temp1 == '-')
		return BAD_SEQUENCE;
	if(temp1[strlen(temp1)-1] == '-')
		return BAD_SEQUENCE;

	stlen = strlen(temp1);
	for(j=0,i=0;i<stlen;i++)
	{
		switch(temp1[i])
		{
			case ' ': case ',':			// prepare for next number
				if((temp1[i-1] != ' ') && (temp1[i-1] != ',')
						&& (temp1[i-1] != '-'))
					j++;
				break;
			case '-':
				if(temp_seqs[j-1].range == -999)
					return BAD_SEQUENCE;
				if((temp1[i-1] != ' ') && (temp1[i-1] != ','))
					temp_seqs[j++].range = -999;  // -999 signifies a range
				else
					temp_seqs[j-1].range = -999;  // -999 signifies a range
				break;
			default:					// it's a number
				temp_seqs[j].range = 0;
				temp_seqs[j].val *= 10;
				temp_seqs[j].val += temp1[i] - 0x30;
				break;
		}
	}

	numvals = j+1;

	// check for out of range numbers
	for(i=0;i<numvals;i++)
	{
		if((temp_seqs[i].val < 1) || (temp_seqs[i].val > max_val))
			return BAD_SEQUENCE;
		if((temp_seqs[i].range == -999) && (temp_seqs[i+1].range == -999))
			return BAD_SEQUENCE;
	}

	for(i=0,j=0;i<numvals;i++)
	{
		if(temp_seqs[i].range == -999)
		{
			if(temp_seqs[i].val < temp_seqs[i+1].val)
				for(k=temp_seqs[i].val;k<=temp_seqs[i+1].val;k++)
					seqs[j++] = k;
			else
				for(k=temp_seqs[i].val;k>=temp_seqs[i+1].val;k--)
					seqs[j++] = k;
			j--;
		}
		else
			seqs[j++] = temp_seqs[i].val;
	}
	numvals = j;
	*NumVals = numvals;
	return GOOD;
} // ConvertStringToInts

int VerifyQuickAnimEntries(char **Entries, Sprite *psprite,
		int **&Sequences, int &NumSequences, int *&SequenceLens)
{
	int num_sprites = psprite->GetNumSprites();
	char entries[50];
	int numvals;
	int rc;
	int seqs[500];
	int j;

	strcpy(entries, Entries[0]);
	rc = ConvertStringToInts(entries, num_sprites, seqs, &numvals);
	if(rc != GOOD)
		return rc;

	NumSequences = 1;
	Sequences = new int *[1];
	SequenceLens = new int[1];
	SequenceLens[0] = numvals;
	Sequences[0] = new int[numvals];
	for(j=0;j<numvals;j++)
		Sequences[0][j] = seqs[j] - 1;
	return GOOD;
} // VerifyQuickAnimEntries

int VerifyCopyEntries(char **Entries, int *dest, int *source, Sprite *ps)
{
	int NumSprites = ps->GetNumSprites();
	int rc1, rc2;

	rc1 = sscanf(Entries[0], "%d", source);
	rc2 = sscanf(Entries[1], "%d", dest);
	if((rc1!=1)||(rc2!=1))
		return BAD;
	(*dest)--;
	(*source)--;
	if((*dest>=0)&&(*dest<NumSprites)&&(*source>=0)&&(*source<NumSprites))
		return GOOD;
	return BAD;
} // VerifyCopyEntries

int VerifyFullAnimEntries(char **Entries, Sprite *psprite,
		int **&Sequences, int &NumSequences, int *&SequenceLens)
{
	int num_sprites = psprite->GetNumSprites();
	char entries[50];
	int rc;
	int seqs[5][500];
	int numvals[5];
	int i,j;
	int num;

	NumSequences = 5;
	for(i=0;i<NumSequences;i++)
	{
		strcpy(entries, Entries[i]);
		rc = ConvertStringToInts(entries, num_sprites, seqs[i],
			&(numvals[i]));
		if(entries[0] == 0)
		{
			numvals[i] = 1;
			seqs[i][0] = 1;
		}
		else if(rc != GOOD)
			return rc;
	}
	NumSequences = 5;
	Sequences = new int *[NumSequences];
	SequenceLens = new int[NumSequences];

	for(num=0;num<NumSequences;num++)
		Sequences[num] = new int[500];
	for(num=0;num<NumSequences;num++)
		for(j=0;j<numvals[num];j++)
			Sequences[num][j] = seqs[num][j] - 1;
	for(num=0;num<NumSequences;num++)
		SequenceLens[num] = numvals[num];
	return GOOD;
} // VerifyFullAnimEntries
